@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ let filterFromCompiledData = function(args) {
 
 // One hostname => one selector
 
-let FilterOneOne = function(hostname, selector) {
+const FilterOneOne = function(hostname, selector) {
     this.hostname = hostname;
     this.selector = selector;
 };
@@ -87,7 +87,7 @@ FilterOneOne.prototype = {
 
     retrieve: function(target, out) {
         if ( target.endsWith(this.hostname) === false ) { return; }
-        let i = target.length - this.hostname.length;
+        const i = target.length - this.hostname.length;
         if ( i !== 0 && target.charCodeAt(i-1) !== 0x2E /* '.' */ ) { return; }
         out.add(this.selector);
     },
@@ -107,7 +107,7 @@ registerFilterClass(FilterOneOne);
 
 // One hostname => many selectors
 
-let FilterOneMany = function(hostname, selectors) {
+const FilterOneMany = function(hostname, selectors) {
     this.hostname = hostname;
     this.selectors = selectors;
 };
@@ -131,7 +131,7 @@ FilterOneMany.prototype = {
 
     retrieve: function(target, out) {
         if ( target.endsWith(this.hostname) === false ) { return; }
-        let i = target.length - this.hostname.length;
+        const i = target.length - this.hostname.length;
         if ( i !== 0 && target.charCodeAt(i-1) !== 0x2E /* '.' */ ) { return; }
         for ( let selector of this.selectors ) {
             out.add(selector);
@@ -161,7 +161,7 @@ FilterManyAny.prototype = {
     fid: 10,
 
     add: function(hostname, selector) {
-        let selectors = this.entries.get(hostname);
+        const selectors = this.entries.get(hostname);
         if ( selectors === undefined ) {
             this.entries.set(hostname, selector);
         } else if ( typeof selectors === 'string' ) {
@@ -172,19 +172,19 @@ FilterManyAny.prototype = {
     },
 
     retrieve: function(target, out) {
-        for ( let entry of this.entries ) {
-            let hostname = entry[0];
+        for ( const entry of this.entries ) {
+            const hostname = entry[0];
             if ( target.endsWith(hostname) === false ) { continue; }
-            let i = target.length - hostname.length;
+            const i = target.length - hostname.length;
             if ( i !== 0 && target.charCodeAt(i-1) !== 0x2E /* '.' */ ) {
                 continue;
             }
-            let selectors = entry[1];
+            const selectors = entry[1];
             if ( typeof selectors === 'string' ) {
                 out.add(selectors);
                 continue;
             }
-            for ( let selector of selectors ) {
+            for ( const selector of selectors ) {
                 out.add(selector);
             }
         }
@@ -338,51 +338,6 @@ SelectorCacheEntry.prototype = {
 /******************************************************************************/
 /******************************************************************************/
 
-// HHHHHHHHHHHH0000
-//            |   |
-//            |   |
-//            |   +-- bit  3-0: reserved: 0=exception
-//            |                           1=procedural
-//            +------ bit 15-4: FNV
-
-let makeHash = function(token) {
-    // Based on: FNV32a
-    // http://www.isthe.com/chongo/tech/comp/fnv/index.html#FNV-reference-source
-    // The rest is custom, suited for uBlock.
-    let i1 = token.length;
-    let i2 = i1 >> 1;
-    let i4 = i1 >> 2;
-    let i8 = i1 >> 3;
-    let hval = (0x811c9dc5 ^ token.charCodeAt(0)) >>> 0;
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i8);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i4);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i4+i8);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i2);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i2+i8);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i2+i4);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-        hval ^= token.charCodeAt(i1-1);
-        hval += (hval<<1) + (hval<<4) + (hval<<7) + (hval<<8) + (hval<<24);
-        hval >>>= 0;
-    return hval & 0xFFF0;
-};
-
-/******************************************************************************/
-/******************************************************************************/
-
 // Cosmetic filter family tree:
 //
 // Generic
@@ -406,25 +361,6 @@ let FilterContainer = function() {
     this.reEscapeSequence = /\\([0-9A-Fa-f]+ |.)/g;
     this.reSimpleHighGeneric1 = /^[a-z]*\[[^[]+]$/;
     this.reHighMedium = /^\[href\^="https?:\/\/([^"]{8})[^"]*"\]$/;
-    this.reNeedHostname = new RegExp([
-        '^',
-        '(?:',
-            [
-            '.+?:has',
-            '.+?:has-text',
-            '.+?:if',
-            '.+?:if-not',
-            '.+?:matches-css(?:-before|-after)?',
-            '.*?:xpath',
-            '.+?:style',
-            '.+?:-abp-contains', // ABP-specific for `:has-text`
-            '.+?:-abp-has',      // ABP-specific for `:if`
-            '.+?:contains'       // Adguard-specific for `:has-text`
-            ].join('|'),
-        ')',
-        '\\(.+\\)',
-        '$'
-    ].join(''));
 
     this.selectorCache = new Map();
     this.selectorCachePruneDelay = 10 * 60 * 1000; // 10 minutes
@@ -538,21 +474,6 @@ FilterContainer.prototype.freeze = function() {
         this.highlyGeneric.simple.dict.size !== 0 ||
         this.highlyGeneric.complex.dict.size !== 0;
 
-    if ( this.genericDonthideSet.size !== 0 ) {
-        for ( let selector of this.genericDonthideSet ) {
-            let type = selector.charCodeAt(0);
-            if ( type === 0x23 /* '#' */ ) {
-                this.lowlyGeneric.id.simple.delete(selector.slice(1));
-            } else if ( type === 0x2E /* '.' */ ) {
-                this.lowlyGeneric.cl.simple.delete(selector.slice(1));
-            }
-            // TODO:
-            //  this.lowlyGeneric.id.complex.delete(selector);
-            //  this.lowlyGeneric.cl.complex.delete(selector);
-            this.highlyGeneric.simple.dict.delete(selector);
-            this.highlyGeneric.complex.dict.delete(selector);
-        }
-    }
     this.highlyGeneric.simple.str = Array.from(this.highlyGeneric.simple.dict).join(',\n');
     this.highlyGeneric.simple.mru.reset();
     this.highlyGeneric.complex.str = Array.from(this.highlyGeneric.complex.dict).join(',\n');
@@ -602,8 +523,8 @@ FilterContainer.prototype.compile = function(parsed, writer) {
     // 1000 = cosmetic filtering
     writer.select(1000);
 
-    let hostnames = parsed.hostnames,
-        i = hostnames.length;
+    const hostnames = parsed.hostnames;
+    let i = hostnames.length;
     if ( i === 0 ) {
         this.compileGenericSelector(parsed, writer);
         return true;
@@ -614,7 +535,7 @@ FilterContainer.prototype.compile = function(parsed, writer) {
     // of same filter OR globally if there is no non-negated hostnames.
     let applyGlobally = true;
     while ( i-- ) {
-        let hostname = hostnames[i];
+        const hostname = hostnames[i];
         if ( hostname.startsWith('~') === false ) {
             applyGlobally = false;
         }
@@ -639,80 +560,72 @@ FilterContainer.prototype.compileGenericSelector = function(parsed, writer) {
 
 /******************************************************************************/
 
-FilterContainer.prototype.compileGenericHideSelector = function(parsed, writer) {
-    let selector = parsed.suffix;
+FilterContainer.prototype.compileGenericHideSelector = function(
+    parsed,
+    writer
+) {
+    const selector = parsed.suffix;
+    const type = selector.charCodeAt(0);
+    let key;
 
-    // For some selectors, it is mandatory to have a hostname or entity:
-    //   ##.foo:-abp-contains(...)
-    //   ##.foo:-abp-has(...)
-    //   ##.foo:contains(...)
-    //   ##.foo:has(...)
-    //   ##.foo:has-text(...)
-    //   ##.foo:if(...)
-    //   ##.foo:if-not(...)
-    //   ##.foo:matches-css(...)
-    //   ##.foo:matches-css-after(...)
-    //   ##.foo:matches-css-before(...)
-    //   ##:xpath(...)
-    //   ##.foo:style(...)
-    if ( this.reNeedHostname.test(selector) ) {
-        µb.logger.writeOne(
-            '',
-            'error',
-            'Cosmetic filtering – invalid generic filter: ##' + selector
+    if ( type === 0x23 /* '#' */ ) {
+        key = this.keyFromSelector(selector);
+        // Simple selector-based CSS rule: no need to test for whether the
+        // selector is valid, the regex took care of this. Most generic
+        // selector falls into that category.
+        // - ###ad-bigbox
+        if ( key === selector ) {
+            writer.push([ 0, key.slice(1) ]);
+            return;
+        }
+    } else if ( type === 0x2E /* '.' */ ) {
+        key = this.keyFromSelector(selector);
+        // Simple selector-based CSS rule: no need to test for whether the
+        // selector is valid, the regex took care of this. Most generic
+        // selector falls into that category.
+        // - ##.ads-bigbox
+        if ( key === selector ) {
+            writer.push([ 2, key.slice(1) ]);
+            return;
+        }
+    }
+
+    const compiled = µb.staticExtFilteringEngine.compileSelector(selector);
+
+    // Invalid cosmetic filter, possible reasons:
+    // - Bad syntax
+    // - Procedural filters (can't be generic): the compiled version of
+    //   a procedural selector is NEVER equal to its raw version.
+    if ( compiled === undefined || compiled !== selector ) {
+        const who = writer.properties.get('assetKey') || '?';
+        µb.logger.writeOne({
+            realm: 'message',
+            type: 'error',
+            text: `Invalid generic cosmetic filter in ${who}: ##${selector}`
+        });
+        return;
+    }
+
+    // Complex selector-based CSS rule:
+    // - ###tads + div + .c
+    // - ##.rscontainer > .ellip
+    if ( key !== undefined ) {
+        writer.push([
+            type === 0x23 /* '#' */ ? 1 : 3,
+            key.slice(1),
+            selector ]
         );
         return;
     }
 
-    let type = selector.charCodeAt(0);
-
-    if ( type === 0x23 /* '#' */ ) {
-        let key = this.keyFromSelector(selector);
-        if ( key === undefined ) { return; }
-        // Simple selector-based CSS rule: no need to test for whether the
-        // selector is valid, the regex took care of this. Most generic
-        // selector falls into that category.
-        if ( key === selector ) {
-            writer.push([ 0 /* lg */, key.slice(1) ]);
-            return;
-        }
-        // Complex selector-based CSS rule.
-        if ( µb.staticExtFilteringEngine.compileSelector(selector) !== undefined ) {
-            writer.push([ 1 /* lg+ */, key.slice(1), selector ]);
-        }
-        return;
-    }
-
-    if ( type === 0x2E /* '.' */ ) {
-        let key = this.keyFromSelector(selector);
-        if ( key === undefined ) { return; }
-        // Simple selector-based CSS rule: no need to test for whether the
-        // selector is valid, the regex took care of this. Most generic
-        // selector falls into that category.
-        if ( key === selector ) {
-            writer.push([ 2 /* lg */, key.slice(1) ]);
-            return;
-        }
-        // Complex selector-based CSS rule.
-        if ( µb.staticExtFilteringEngine.compileSelector(selector) !== undefined ) {
-            writer.push([ 3 /* lg+ */, key.slice(1), selector ]);
-        }
-        return;
-    }
-
-    let compiled = µb.staticExtFilteringEngine.compileSelector(selector);
-    if ( compiled === undefined ) { return; }
-    // TODO: Detect and error on procedural cosmetic filters.
-
     // https://github.com/gorhill/uBlock/issues/909
     //   Anything which contains a plain id/class selector can be classified
     //   as a low generic cosmetic filter.
-    let matches = this.rePlainSelectorEx.exec(selector);
+    const matches = this.rePlainSelectorEx.exec(selector);
     if ( matches !== null ) {
-        let key = matches[1] || matches[2];
-        type = key.charCodeAt(0);
+        const key = matches[1] || matches[2];
         writer.push([
-            type === 0x23 ? 1 : 3 /* lg+ */,
+            key.charCodeAt(0) === 0x23 /* '#' */ ? 1 : 3,
             key.slice(1),
             selector
         ]);
@@ -744,7 +657,15 @@ FilterContainer.prototype.compileGenericUnhideSelector = function(
 ) {
     // Procedural cosmetic filters are acceptable as generic exception filters.
     let compiled = µb.staticExtFilteringEngine.compileSelector(parsed.suffix);
-    if ( compiled === undefined ) { return; }
+    if ( compiled === undefined ) {
+        const who = writer.properties.get('assetKey') || '?';
+        µb.logger.writeOne({
+            realm: 'message',
+            type: 'error',
+            text: `Invalid cosmetic filter in ${who}: #@#${parsed.suffix}`
+        });
+        return;
+    }
 
     // https://github.com/chrisaljoudi/uBlock/issues/497
     //   All generic exception filters are put in the same bucket: they are
@@ -767,27 +688,29 @@ FilterContainer.prototype.compileSpecificSelector = function(
     }
 
     let compiled = µb.staticExtFilteringEngine.compileSelector(parsed.suffix);
-    if ( compiled === undefined ) { return; }
-
-    // https://github.com/chrisaljoudi/uBlock/issues/188
-    // If not a real domain as per PSL, assign a synthetic one
-    let hash;
-    if ( hostname.endsWith('.*') === false ) {
-        let domain = this.µburi.domainFromHostnameNoCache(hostname);
-        hash = domain !== '' ? makeHash(domain) : 0;
-    } else {
-        hash = makeHash(hostname);
+    if ( compiled === undefined ) {
+        const who = writer.properties.get('assetKey') || '?';
+        µb.logger.writeOne({
+            realm: 'message',
+            type: 'error',
+            text: `Invalid cosmetic filter in ${who}: ##${parsed.suffix}`
+        });
+        return;
     }
+
+    let hash = µb.staticExtFilteringEngine.compileHostnameToHash(hostname);
+
+    // Exception?
     if ( unhide === 1 ) {
-        hash |= 0b01;
+        hash |= 0b0001;
     }
 
-    writer.push([
-        8,
-        compiled.charCodeAt(0) !== 0x7B ? hash : hash | 0b10,
-        hostname,
-        compiled
-    ]);
+    // Procedural?
+    if ( compiled.charCodeAt(0) === 0x7B ) {
+        hash |= 0b0010;
+    }
+
+    writer.push([ 8, hash, hostname, compiled ]);
 };
 
 /******************************************************************************/
@@ -828,9 +751,9 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
             if ( bucket === undefined ) {
                 db.simple.add(args[1]);
             } else if ( Array.isArray(bucket) ) {
-                bucket.push(args[1]);
+                bucket.push(db.prefix + args[1]);
             } else {
-                db.complex.set(args[1], [ bucket, args[1] ]);
+                db.complex.set(args[1], [ bucket, db.prefix + args[1] ]);
             }
             break;
 
@@ -841,7 +764,7 @@ FilterContainer.prototype.fromCompiledContent = function(reader, options) {
             bucket = db.complex.get(args[1]);
             if ( bucket === undefined ) {
                 if ( db.simple.has(args[1]) ) {
-                    db.complex.set(args[1], [ args[1], args[2] ]);
+                    db.complex.set(args[1], [ db.prefix + args[1], args[2] ]);
                 } else {
                     db.complex.set(args[1], args[2]);
                     db.simple.add(args[1]);
@@ -904,17 +827,15 @@ FilterContainer.prototype.skipGenericCompiledContent = function(reader) {
     // 1000 = cosmetic filtering
     reader.select(1000);
 
-    let bucket;
-
     while ( reader.next() ) {
         this.acceptedCount += 1;
-        let fingerprint = reader.fingerprint();
+        const fingerprint = reader.fingerprint();
         if ( this.duplicateBuster.has(fingerprint) ) {
             this.discardedCount += 1;
             continue;
         }
 
-        let args = reader.args();
+        const args = reader.args();
 
         switch ( args[0] ) {
 
@@ -928,7 +849,8 @@ FilterContainer.prototype.skipGenericCompiledContent = function(reader) {
         // hash,  example.com, .promoted-tweet
         // hash,  example.*, .promoted-tweet
         case 8:
-            bucket = this.specificFilters.get(args[1]);
+            this.duplicateBuster.add(fingerprint);
+            const bucket = this.specificFilters.get(args[1]);
             if ( bucket === undefined ) {
                 this.specificFilters.set(
                     args[1],
@@ -1138,14 +1060,8 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
     for ( let type in this.lowlyGeneric ) {
         let entry = this.lowlyGeneric[type];
         let selectors = request[entry.canonical];
-        if ( typeof selectors !== 'string' ) { continue; }
-        let strEnd = selectors.length;
-        let sliceBeg = 0;
-        do {
-            let sliceEnd = selectors.indexOf('\n', sliceBeg);
-            if ( sliceEnd === -1 ) { sliceEnd = strEnd; }
-            let selector = selectors.slice(sliceBeg, sliceEnd);
-            sliceBeg = sliceEnd + 1;
+        if ( Array.isArray(selectors) === false ) { continue; }
+        for ( let selector of selectors ) {
             if ( entry.simple.has(selector) === false ) { continue; }
             let bucket = entry.complex.get(selector);
             if ( bucket !== undefined ) {
@@ -1164,7 +1080,7 @@ FilterContainer.prototype.retrieveGenericSelectors = function(request) {
                     simpleSelectors.add(selector);
                 }
             }
-        } while ( sliceBeg < strEnd );
+        }
     }
 
     // Apply exceptions: it is the responsibility of the caller to provide
@@ -1268,8 +1184,9 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
 
     if ( options.noCosmeticFiltering !== true ) {
         let entity = request.entity,
-            domainHash = makeHash(request.domain),
-            entityHash = entity !== '' ? makeHash(entity) : undefined;
+            domainHash = µb.staticExtFilteringEngine.makeHash(request.domain),
+            entityHash = µb.staticExtFilteringEngine.makeHash(entity),
+            bucket;
 
         // Exception cosmetic filters: prime with generic exception filters.
         let exceptionSet = this.setRegister0;
@@ -1278,32 +1195,34 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
             exceptionSet.add(exception);
         }
         // Specific exception cosmetic filters.
-        let bucket = this.specificFilters.get(domainHash | 0b01);
-        if ( bucket !== undefined ) {
-            bucket.retrieve(hostname, exceptionSet);
-        }
-        bucket = this.specificFilters.get(domainHash | 0b11);
-        if ( bucket !== undefined ) {
-            bucket.retrieve(hostname, exceptionSet);
+        if ( domainHash !== 0 ) {
+            bucket = this.specificFilters.get(domainHash | 0b0001);
+            if ( bucket !== undefined ) {
+                bucket.retrieve(hostname, exceptionSet);
+            }
+            bucket = this.specificFilters.get(domainHash | 0b0011);
+            if ( bucket !== undefined ) {
+                bucket.retrieve(hostname, exceptionSet);
+            }
         }
         // Specific entity-based exception cosmetic filters.
-        if ( entityHash !== undefined ) {
-            bucket = this.specificFilters.get(entityHash | 0b01);
+        if ( entityHash !== 0 ) {
+            bucket = this.specificFilters.get(entityHash | 0b0001);
             if ( bucket !== undefined ) {
                 bucket.retrieve(entity, exceptionSet);
             }
-            bucket = this.specificFilters.get(entityHash | 0b11);
+            bucket = this.specificFilters.get(entityHash | 0b0011);
             if ( bucket !== undefined ) {
                 bucket.retrieve(entity, exceptionSet);
             }
         }
         // Special bucket for those filters without a valid
         // domain name as per PSL.
-        bucket = this.specificFilters.get(0b01);
+        bucket = this.specificFilters.get(0 | 0b0001);
         if ( bucket !== undefined ) {
             bucket.retrieve(hostname, exceptionSet);
         }
-        bucket = this.specificFilters.get(0b11);
+        bucket = this.specificFilters.get(0 | 0b0011);
         if ( bucket !== undefined ) {
             bucket.retrieve(hostname, exceptionSet);
         }
@@ -1317,20 +1236,23 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         //       slightly content script code.
         let specificSet = this.setRegister1;
         // Specific cosmetic filters.
-        bucket = this.specificFilters.get(domainHash | 0b00);
-        if ( bucket !== undefined ) {
-            bucket.retrieve(hostname, specificSet);
+        if ( domainHash !== 0 ) {
+            bucket = this.specificFilters.get(domainHash | 0b0000);
+            if ( bucket !== undefined ) {
+                bucket.retrieve(hostname, specificSet);
+            }
         }
         // Specific entity-based cosmetic filters.
-        if ( entityHash !== undefined ) {
-            bucket = this.specificFilters.get(entityHash | 0b00);
+        if ( entityHash !== 0 ) {
+            bucket = this.specificFilters.get(entityHash | 0b0000);
             if ( bucket !== undefined ) {
                 bucket.retrieve(entity, specificSet);
             }
         }
         // https://github.com/chrisaljoudi/uBlock/issues/188
-        // Special bucket for those filters without a valid domain name as per PSL
-        bucket = this.specificFilters.get(0b00);
+        //   Special bucket for those filters without a valid domain name
+        //   as per PSL
+        bucket = this.specificFilters.get(0 | 0b0000);
         if ( bucket !== undefined ) {
             bucket.retrieve(hostname, specificSet);
         }
@@ -1346,20 +1268,23 @@ FilterContainer.prototype.retrieveSpecificSelectors = function(
         // Procedural cosmetic filters.
         let proceduralSet = this.setRegister2;
         // Specific cosmetic filters.
-        bucket = this.specificFilters.get(domainHash | 0b10);
-        if ( bucket !== undefined ) {
-            bucket.retrieve(hostname, proceduralSet);
+        if ( domainHash !== 0 ) {
+            bucket = this.specificFilters.get(domainHash | 0b0010);
+            if ( bucket !== undefined ) {
+                bucket.retrieve(hostname, proceduralSet);
+            }
         }
         // Specific entity-based cosmetic filters.
-        if ( entityHash !== undefined ) {
-            bucket = this.specificFilters.get(entityHash | 0b10);
+        if ( entityHash !== 0 ) {
+            bucket = this.specificFilters.get(entityHash | 0b0010);
             if ( bucket !== undefined ) {
                 bucket.retrieve(entity, proceduralSet);
             }
         }
         // https://github.com/chrisaljoudi/uBlock/issues/188
-        // Special bucket for those filters without a valid domain name as per PSL
-        bucket = this.specificFilters.get(0b10);
+        //   Special bucket for those filters without a valid domain name
+        //   as per PSL
+        bucket = this.specificFilters.get(0 | 0b0010);
         if ( bucket !== undefined ) {
             bucket.retrieve(hostname, proceduralSet);
         }
