@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ var getDomainNames = function(targets) {
 
 /******************************************************************************/
 
-var onMessage = function(request, sender, callback) {
+const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     case 'getAssetContent':
@@ -69,11 +69,7 @@ var onMessage = function(request, sender, callback) {
         return;
 
     case 'listsFromCosmeticFilter':
-        µb.staticFilteringReverseLookup.fromCosmeticFilter(
-            request.hostname,
-            request.rawFilter,
-            callback
-        );
+        µb.staticFilteringReverseLookup.fromCosmeticFilter(request, callback);
         return;
 
     case 'reloadAllFilters':
@@ -88,10 +84,7 @@ var onMessage = function(request, sender, callback) {
         break;
     }
 
-    // The concatenation with the empty string ensure that the resulting value
-    // is a string. This is important since tab id values are assumed to be
-    // of string type.
-    var tabId = sender && sender.tab ? '' + sender.tab.id : 0;
+    const tabId = sender && sender.tab ? sender.tab.id : 0;
 
     // Sync
     var response;
@@ -110,7 +103,7 @@ var onMessage = function(request, sender, callback) {
         break;
 
     case 'createUserFilter':
-        µb.appendUserFilters(request.filters);
+        µb.appendUserFilters(request.filters, request);
         // https://github.com/gorhill/uBlock/issues/1786
         µb.cosmeticFilteringEngine.removeFromSelectorCache(request.pageDomain);
         break;
@@ -286,9 +279,10 @@ var getFirewallRules = function(srcHostname, desHostnames) {
 /******************************************************************************/
 
 var popupDataFromTabId = function(tabId, tabTitle) {
-    var tabContext = µb.tabContextManager.mustLookup(tabId),
+    let tabContext = µb.tabContextManager.mustLookup(tabId),
         rootHostname = tabContext.rootHostname;
-    var r = {
+
+    let r = {
         advancedUserEnabled: µb.userSettings.advancedUserEnabled,
         appName: vAPI.app.name,
         appVersion: vAPI.app.version,
@@ -312,7 +306,7 @@ var popupDataFromTabId = function(tabId, tabTitle) {
         tooltipsDisabled: µb.userSettings.tooltipsDisabled
     };
 
-    var pageStore = µb.pageStoreFromTabId(tabId);
+    let pageStore = µb.pageStoreFromTabId(tabId);
     if ( pageStore ) {
         // https://github.com/gorhill/uBlock/issues/2105
         //   Be sure to always include the current page's hostname -- it might
@@ -332,22 +326,30 @@ var popupDataFromTabId = function(tabId, tabTitle) {
         r.contentLastModified = pageStore.contentLastModified;
         r.firewallRules = getFirewallRules(rootHostname, r.hostnameDict);
         r.canElementPicker = µb.URI.isNetworkURI(r.rawURL);
-        r.noPopups = µb.hnSwitches.evaluateZ('no-popups', rootHostname);
+        r.noPopups = µb.sessionSwitches.evaluateZ('no-popups', rootHostname);
         r.popupBlockedCount = pageStore.popupBlockedCount;
-        r.noCosmeticFiltering = µb.hnSwitches.evaluateZ('no-cosmetic-filtering', rootHostname);
-        r.noLargeMedia = µb.hnSwitches.evaluateZ('no-large-media', rootHostname);
+        r.noCosmeticFiltering = µb.sessionSwitches.evaluateZ('no-cosmetic-filtering', rootHostname);
+        r.noLargeMedia = µb.sessionSwitches.evaluateZ('no-large-media', rootHostname);
         r.largeMediaCount = pageStore.largeMediaCount;
-        r.noRemoteFonts = µb.hnSwitches.evaluateZ('no-remote-fonts', rootHostname);
+        r.noRemoteFonts = µb.sessionSwitches.evaluateZ('no-remote-fonts', rootHostname);
         r.remoteFontCount = pageStore.remoteFontCount;
+        r.noScripting = µb.sessionSwitches.evaluateZ('no-scripting', rootHostname);
     } else {
         r.hostnameDict = {};
         r.firewallRules = getFirewallRules();
     }
-    r.matrixIsDirty = !µb.sessionFirewall.hasSameRules(
+
+    r.matrixIsDirty = µb.sessionFirewall.hasSameRules(
         µb.permanentFirewall,
         rootHostname,
         r.hostnameDict
-    );
+    ) === false;
+    if ( r.matrixIsDirty === false ) {
+        r.matrixIsDirty = µb.sessionSwitches.hasSameRules(
+            µb.permanentSwitches,
+            rootHostname
+        ) === false;
+    }
     return r;
 };
 
@@ -378,14 +380,6 @@ var onMessage = function(request, sender, callback) {
 
     // Async
     switch ( request.what ) {
-    case 'getPopupLazyData':
-        pageStore = µb.pageStoreFromTabId(request.tabId);
-        if ( pageStore !== null ) {
-            pageStore.hiddenElementCount = 0;
-            µb.scriptlets.injectDeep(request.tabId, 'cosmetic-survey');
-        }
-        return;
-
     case 'getPopupData':
         popupDataFromRequest(request, callback);
         return;
@@ -398,6 +392,19 @@ var onMessage = function(request, sender, callback) {
     var response;
 
     switch ( request.what ) {
+    case 'getPopupLazyData':
+        pageStore = µb.pageStoreFromTabId(request.tabId);
+        if ( pageStore !== null ) {
+            pageStore.hiddenElementCount = 0;
+            pageStore.scriptCount = 0;
+            vAPI.tabs.injectScript(request.tabId, {
+                allFrames: true,
+                file: '/js/scriptlets/dom-survey.js',
+                runAt: 'document_end'
+            });
+        }
+        break;
+
     case 'hasPopupContentChanged':
         pageStore = µb.pageStoreFromTabId(request.tabId);
         var lastModified = pageStore ? pageStore.contentLastModified : 0;
@@ -410,18 +417,41 @@ var onMessage = function(request, sender, callback) {
             request.srcHostname,
             request.desHostnames
         );
+        µb.sessionSwitches.copyRules(
+            µb.permanentSwitches,
+            request.srcHostname
+        );
         // https://github.com/gorhill/uBlock/issues/188
-        µb.cosmeticFilteringEngine.removeFromSelectorCache(request.srcHostname, 'net');
+        µb.cosmeticFilteringEngine.removeFromSelectorCache(
+            request.srcHostname,
+            'net'
+        );
         response = popupDataFromTabId(request.tabId);
         break;
 
     case 'saveFirewallRules':
-        µb.permanentFirewall.copyRules(
-            µb.sessionFirewall,
-            request.srcHostname,
-            request.desHostnames
-        );
-        µb.savePermanentFirewallRules();
+        if (
+            µb.permanentFirewall.copyRules(
+                µb.sessionFirewall,
+                request.srcHostname,
+                request.desHostnames
+            )
+        ) {
+            µb.savePermanentFirewallRules();
+        }
+        if (
+            µb.permanentSwitches.copyRules(
+                µb.sessionSwitches,
+                request.srcHostname
+            )
+        ) {
+            µb.saveHostnameSwitches();
+        }
+        break;
+
+    case 'toggleHostnameSwitch':
+        µb.toggleHostnameSwitch(request);
+        response = popupDataFromTabId(request.tabId);
         break;
 
     case 'toggleFirewallRule':
@@ -467,8 +497,8 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var µb = µBlock,
-        response,
+    const µb = µBlock;
+    let response,
         tabId, frameId,
         pageStore = null;
 
@@ -495,6 +525,21 @@ var onMessage = function(request, sender, callback) {
         }
         break;
 
+    case 'shouldRenderNoscriptTags':
+        if ( pageStore === null ) { break; }
+        const fctxt = µb.filteringContext.fromTabId(tabId);
+        if ( pageStore.filterScripting(fctxt, undefined) ) {
+            vAPI.tabs.injectScript(
+                tabId,
+                {
+                    file: '/js/scriptlets/noscript-spoof.js',
+                    frameId: frameId,
+                    runAt: 'document_end'
+                }
+            );
+        }
+        break;
+
     case 'retrieveContentScriptParameters':
         if (
             pageStore === null ||
@@ -515,12 +560,18 @@ var onMessage = function(request, sender, callback) {
         request.domain = µb.URI.domainFromHostname(request.hostname);
         request.entity = µb.URI.entityFromDomain(request.domain);
         response.specificCosmeticFilters =
-            µb.cosmeticFilteringEngine.retrieveSpecificSelectors(request, response);
+            µb.cosmeticFilteringEngine.retrieveSpecificSelectors(
+                request,
+                response
+            );
         if ( µb.canInjectScriptletsNow === false ) {
             response.scriptlets = µb.scriptletFilteringEngine.retrieve(request);
         }
-        if ( response.noCosmeticFiltering !== true ) {
-            µb.logCosmeticFilters(tabId, frameId);
+        if ( µb.logger.enabled ) {
+            if ( response.noCosmeticFiltering !== true ) {
+                µb.logCosmeticFilters(tabId, frameId);
+            }
+            µb.logInlineScript(tabId, frameId);
         }
         break;
 
@@ -557,15 +608,13 @@ vAPI.messaging.listen('contentscript', onMessage);
 
 /******************************************************************************/
 
-var µb = µBlock;
+const onMessage = function(request, sender, callback) {
+    const µb = µBlock;
 
-/******************************************************************************/
-
-var onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     case 'elementPickerArguments':
-        var xhr = new XMLHttpRequest();
+        const xhr = new XMLHttpRequest();
         xhr.open('GET', 'epicker.html', true);
         xhr.overrideMimeType('text/html;charset=utf-8');
         xhr.responseType = 'text';
@@ -581,8 +630,8 @@ var onMessage = function(request, sender, callback) {
                 cosmeticFilters: vAPI.i18n('pickerCosmeticFilters'),
                 cosmeticFiltersHint: vAPI.i18n('pickerCosmeticFiltersHint')
             };
-            var reStrings = /\{\{(\w+)\}\}/g;
-            var replacer = function(a0, string) {
+            const reStrings = /\{\{(\w+)\}\}/g;
+            const replacer = function(a0, string) {
                 return i18n[string];
             };
 
@@ -606,7 +655,7 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
     case 'elementPickerEprom':
@@ -697,7 +746,7 @@ vAPI.messaging.listen('cloudWidget', onMessage);
 
 /******************************************************************************/
 
-var µb = µBlock;
+const µb = µBlock;
 
 /******************************************************************************/
 
@@ -730,7 +779,7 @@ var backupUserData = function(callback) {
         netWhitelist: µb.stringFromWhitelist(µb.netWhitelist),
         dynamicFilteringString: µb.permanentFirewall.toString(),
         urlFilteringString: µb.permanentURLFiltering.toString(),
-        hostnameSwitchesString: µb.hnSwitches.toString(),
+        hostnameSwitchesString: µb.permanentSwitches.toString(),
         userFilters: ''
     };
 
@@ -791,20 +840,26 @@ var restoreUserData = function(request) {
 
     // If we are going to restore all, might as well wipe out clean local
     // storage
-    vAPI.cacheStorage.clear();
+    µb.cacheStorage.clear();
     vAPI.storage.clear(onAllRemoved);
     vAPI.localStorage.removeItem('immediateHiddenSettings');
 };
 
+// Remove all stored data but keep global counts, people can become
+// quite attached to numbers
+
 var resetUserData = function() {
-    vAPI.cacheStorage.clear();
-    vAPI.storage.clear();
+    let count = 3;
+    let countdown = ( ) => {
+        count -= 1;
+        if ( count === 0 ) {
+            vAPI.app.restart();
+        }
+    };
+    µb.cacheStorage.clear(countdown); // 1
+    vAPI.storage.clear(countdown);    // 2
+    µb.saveLocalSettings(countdown);  // 3
     vAPI.localStorage.removeItem('immediateHiddenSettings');
-
-    // Keep global counts, people can become quite attached to numbers
-    µb.saveLocalSettings();
-
-    vAPI.app.restart();
 };
 
 /******************************************************************************/
@@ -859,44 +914,55 @@ var getLists = function(callback) {
 
 var getRules = function() {
     return {
-        permanentRules: µb.permanentFirewall.toArray().concat(
-                            µb.permanentURLFiltering.toArray()
-                        ),
-          sessionRules: µb.sessionFirewall.toArray().concat(
-                            µb.sessionURLFiltering.toArray()
-                        ),
-            hnSwitches: µb.hnSwitches.toArray()
+        permanentRules:
+            µb.permanentFirewall.toArray().concat(
+                µb.permanentSwitches.toArray(),
+                µb.permanentURLFiltering.toArray()
+            ),
+        sessionRules:
+            µb.sessionFirewall.toArray().concat(
+                µb.sessionSwitches.toArray(),
+                µb.sessionURLFiltering.toArray()
+            )
     };
 };
 
 var modifyRuleset = function(details) {
-    var swRuleset = µb.hnSwitches,
-        hnRuleset, urlRuleset;
+    let swRuleset, hnRuleset, urlRuleset;
     if ( details.permanent ) {
+        swRuleset = µb.permanentSwitches;
         hnRuleset = µb.permanentFirewall;
         urlRuleset = µb.permanentURLFiltering;
     } else {
+        swRuleset = µb.sessionSwitches;
         hnRuleset = µb.sessionFirewall;
         urlRuleset = µb.sessionURLFiltering;
     }
-    var toRemove = new Set(details.toRemove.trim().split(/\s*[\n\r]+\s*/));
-    var rule, parts, _;
-    for ( rule of toRemove ) {
+    let toRemove = new Set(details.toRemove.trim().split(/\s*[\n\r]+\s*/));
+    for ( let rule of toRemove ) {
         if ( rule === '' ) { continue; }
-        parts = rule.split(/\s+/);
-        _ = hnRuleset.removeFromRuleParts(parts) ||
-            swRuleset.removeFromRuleParts(parts) ||
-            urlRuleset.removeFromRuleParts(parts);
+        let parts = rule.split(/\s+/);
+        if ( hnRuleset.removeFromRuleParts(parts) === false ) {
+            if ( swRuleset.removeFromRuleParts(parts) === false ) {
+                urlRuleset.removeFromRuleParts(parts);
+            }
+        }
     }
-    var toAdd = new Set(details.toAdd.trim().split(/\s*[\n\r]+\s*/));
-    for ( rule of toAdd ) {
+    let toAdd = new Set(details.toAdd.trim().split(/\s*[\n\r]+\s*/));
+    for ( let rule of toAdd ) {
         if ( rule === '' ) { continue; }
-        parts = rule.split(/\s+/);
-        _ = hnRuleset.addFromRuleParts(parts) ||
-            swRuleset.addFromRuleParts(parts) ||
-            urlRuleset.addFromRuleParts(parts);
+        let parts = rule.split(/\s+/);
+        if ( hnRuleset.addFromRuleParts(parts) === false ) {
+            if ( swRuleset.addFromRuleParts(parts) === false ) {
+                urlRuleset.addFromRuleParts(parts);
+            }
+        }
     }
     if ( details.permanent ) {
+        if ( swRuleset.changed ) {
+            µb.saveHostnameSwitches();
+            swRuleset.changed = false;
+        }
         if ( hnRuleset.changed ) {
             µb.savePermanentFirewallRules();
             hnRuleset.changed = false;
@@ -906,10 +972,43 @@ var modifyRuleset = function(details) {
             urlRuleset.changed = false;
         }
     }
-    if ( swRuleset.changed ) {
-        µb.saveHostnameSwitches();
-        swRuleset.changed = false;
+};
+
+/******************************************************************************/
+
+// Shortcuts pane
+
+let getShortcuts = function(callback) {
+    if ( µb.canUseShortcuts === false ) {
+        return callback([]);
     }
+
+    vAPI.commands.getAll(commands => {
+        let response = [];
+        for ( let command of commands ) {
+            let desc = command.description;
+            let match = /^__MSG_(.+?)__$/.exec(desc);
+            if ( match !== null ) {
+                desc = vAPI.i18n(match[1]);
+            }
+            if ( desc === '' ) { continue; }
+            command.description = desc;
+            response.push(command);
+        }
+        callback(response);
+    });
+};
+
+let setShortcut = function(details) {
+    if  ( µb.canUpdateShortcuts === false ) { return; }
+    if ( details.shortcut === undefined ) {
+        vAPI.commands.reset(details.name);
+        µb.commandShortcuts.delete(details.name);
+    } else {
+        vAPI.commands.update({ name: details.name, shortcut: details.shortcut });
+        µb.commandShortcuts.set(details.name, details.shortcut);
+    }
+    vAPI.storage.set({ commandShortcuts: Array.from(µb.commandShortcuts) });
 };
 
 /******************************************************************************/
@@ -926,6 +1025,9 @@ var onMessage = function(request, sender, callback) {
     case 'getLocalData':
         return getLocalData(callback);
 
+    case 'getShortcuts':
+        return getShortcuts(callback);
+
     case 'readUserFilters':
         return µb.loadUserFilters(callback);
 
@@ -940,6 +1042,10 @@ var onMessage = function(request, sender, callback) {
     var response;
 
     switch ( request.what ) {
+    case 'canUpdateShortcuts':
+        response = µb.canUpdateShortcuts;
+        break;
+
     case 'getRules':
         response = getRules();
         break;
@@ -981,6 +1087,10 @@ var onMessage = function(request, sender, callback) {
         resetUserData();
         break;
 
+    case 'setShortcut':
+        setShortcut(request);
+        break;
+
     case 'writeHiddenSettings':
         µb.changeHiddenSettings(µb.hiddenSettingsFromString(request.content));
         break;
@@ -1007,30 +1117,31 @@ vAPI.messaging.listen('dashboard', onMessage);
 
 /******************************************************************************/
 
-var µb = µBlock,
-    extensionOriginURL = vAPI.getURL('');
+const µb = µBlock;
+const extensionOriginURL = vAPI.getURL('');
 
 /******************************************************************************/
 
-var getLoggerData = function(details, activeTabId, callback) {
-    let response = {
+const getLoggerData = function(details, activeTabId, callback) {
+    const response = {
         colorBlind: µb.userSettings.colorBlindFriendly,
         entries: µb.logger.readAll(details.ownerId),
         maxEntries: µb.userSettings.requestLogMaxEntries,
         activeTabId: activeTabId,
-        tabIdsToken: µb.pageStoresToken
+        tabIdsToken: µb.pageStoresToken,
+        tooltips: µb.userSettings.tooltipsDisabled === false
     };
     if ( µb.pageStoresToken !== details.tabIdsToken ) {
-        let tabIds = new Map();
-        for ( let entry of µb.pageStores ) {
-            let pageStore = entry[1];
+        const tabIds = new Map();
+        for ( const entry of µb.pageStores ) {
+            const pageStore = entry[1];
             if ( pageStore.rawURL.startsWith(extensionOriginURL) ) { continue; }
             tabIds.set(entry[0], pageStore.title);
         }
         response.tabIds = Array.from(tabIds);
     }
     if ( activeTabId ) {
-        let pageStore = µb.pageStoreFromTabId(activeTabId);
+        const pageStore = µb.pageStoreFromTabId(activeTabId);
         if (
             pageStore === null ||
             pageStore.rawURL.startsWith(extensionOriginURL)
@@ -1038,43 +1149,67 @@ var getLoggerData = function(details, activeTabId, callback) {
             response.activeTabId = undefined;
         }
     }
+    if ( details.popupLoggerBoxChanged && browser.windows instanceof Object ) {
+        browser.tabs.query(
+            { url: vAPI.getURL('/logger-ui.html?popup=1') },
+            tabs => {
+                if ( Array.isArray(tabs) === false ) { return; }
+                if ( tabs.length === 0 ) { return; }
+                browser.windows.get(tabs[0].windowId, win => {
+                    if ( win instanceof Object === false ) { return; }
+                    vAPI.localStorage.setItem(
+                        'popupLoggerBox',
+                        JSON.stringify({
+                            left: win.left,
+                            top: win.top,
+                            width: win.width,
+                            height: win.height,
+                        })
+                    );
+                });
+            }
+        );
+    }
     callback(response);
 };
 
 /******************************************************************************/
 
-var getURLFilteringData = function(details) {
-    var colors = {};
-    var response = {
+const getURLFilteringData = function(details) {
+    const colors = {};
+    const response = {
         dirty: false,
         colors: colors
     };
-    var suf = µb.sessionURLFiltering;
-    var puf = µb.permanentURLFiltering;
-    var urls = details.urls,
-        context = details.context,
-        type = details.type;
-    var url, colorEntry;
-    var i = urls.length;
-    while ( i-- ) {
-        url = urls[i];
-        colorEntry = colors[url] = { r: 0, own: false };
+    const suf = µb.sessionURLFiltering;
+    const puf = µb.permanentURLFiltering;
+    const urls = details.urls;
+    const context = details.context;
+    const type = details.type;
+    for ( const url of urls ) {
+        const colorEntry = colors[url] = { r: 0, own: false };
         if ( suf.evaluateZ(context, url, type).r !== 0 ) {
             colorEntry.r = suf.r;
-            colorEntry.own = suf.r !== 0 && suf.context === context && suf.url === url && suf.type === type;
-        }
-        if ( response.dirty ) {
-            continue;
+            colorEntry.own = suf.r !== 0 &&
+                             suf.context === context &&
+                             suf.url === url &&
+                             suf.type === type;
         }
+        if ( response.dirty ) { continue; }
         puf.evaluateZ(context, url, type);
-        response.dirty = colorEntry.own !== (puf.r !== 0 && puf.context === context && puf.url === url && puf.type === type);
+        response.dirty = colorEntry.own !== (
+            puf.r !== 0 &&
+            puf.context === context &&
+            puf.url === url &&
+            puf.type === type
+        );
     }
     return response;
 };
 
 /******************************************************************************/
 
-var onMessage = function(request, sender, callback) {
+const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     case 'readAll':
@@ -1095,7 +1230,7 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
     case 'releaseView':
@@ -1154,11 +1289,11 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
     case 'temporarilyWhitelistDocument':
-        µBlock.webRequest.temporarilyWhitelistDocument(request.hostname);
+        µBlock.webRequest.strictBlockBypass(request.hostname);
         break;
 
     default:
@@ -1183,55 +1318,49 @@ vAPI.messaging.listen('documentBlocked', onMessage);
 
 /******************************************************************************/
 
-var µb = µBlock;
-var broadcastTimers = Object.create(null);
+const µb = µBlock;
+const broadcastTimers = new Map();
 
 /******************************************************************************/
 
-var cosmeticallyFilteredElementCountChanged = function(tabId) {
-    delete broadcastTimers[tabId + '-cosmeticallyFilteredElementCountChanged'];
+const domSurveyFinalReport = function(tabId) {
+    broadcastTimers.delete(tabId + '-domSurveyReport');
 
-    var pageStore = µb.pageStoreFromTabId(tabId);
-    if ( pageStore === null ) {
-        return;
-    }
+    const pageStore = µb.pageStoreFromTabId(tabId);
+    if ( pageStore === null ) { return; }
 
     vAPI.messaging.broadcast({
-        what: 'cosmeticallyFilteredElementCountChanged',
+        what: 'domSurveyFinalReport',
         tabId: tabId,
-        count: pageStore.hiddenElementCount
+        affectedElementCount: pageStore.hiddenElementCount,
+        scriptCount: pageStore.scriptCount,
     });
 };
 
 /******************************************************************************/
 
-var logCosmeticFilters = function(tabId, details) {
-    if ( µb.logger.isEnabled() === false ) {
-        return;
-    }
+const logCosmeticFilters = function(tabId, details) {
+    if ( µb.logger.enabled === false ) { return; }
 
-    var selectors = details.matchedSelectors;
-
-    selectors.sort();
-
-    for ( var i = 0; i < selectors.length; i++ ) {
-        µb.logger.writeOne(
-            tabId,
-            'cosmetic',
-            { source: 'cosmetic', raw: '##' + selectors[i] },
-            'dom',
-            details.frameURL,
-            null,
-            details.frameHostname
-        );
+    const filter = { source: 'cosmetic', raw: '' };
+    const fctxt = µb.filteringContext.duplicate();
+    fctxt.fromTabId(tabId)
+         .setRealm('cosmetic')
+         .setType('dom')
+         .setURL(details.frameURL)
+         .setDocOriginFromURL(details.frameURL)
+         .setFilter(filter);
+    for ( const selector of details.matchedSelectors.sort() ) {
+        filter.raw = '##' + selector;
+        fctxt.toLogger();
     }
 };
 
 /******************************************************************************/
 
 var onMessage = function(request, sender, callback) {
-    var tabId = sender && sender.tab ? sender.tab.id : 0;
-    var pageStore = µb.pageStoreFromTabId(tabId);
+    const tabId = sender && sender.tab ? sender.tab.id : 0;
+    const pageStore = µb.pageStoreFromTabId(tabId);
 
     // Async
     switch ( request.what ) {
@@ -1243,15 +1372,33 @@ var onMessage = function(request, sender, callback) {
     var response;
 
     switch ( request.what ) {
-    case 'cosmeticallyFilteredElementCount':
-        if ( pageStore !== null && request.filteredElementCount ) {
-            pageStore.hiddenElementCount += request.filteredElementCount;
-            var broadcastKey = tabId + '-cosmeticallyFilteredElementCountChanged';
-            if ( broadcastTimers[broadcastKey] === undefined ) {
-                broadcastTimers[broadcastKey] = vAPI.setTimeout(
-                    cosmeticallyFilteredElementCountChanged.bind(null, tabId),
-                    250
-                );
+    case 'domSurveyTransientReport':
+        if ( pageStore !== null ) {
+            if ( request.filteredElementCount ) {
+                pageStore.hiddenElementCount += request.filteredElementCount;
+            }
+            if ( request.scriptCount ) {
+                pageStore.scriptCount += request.scriptCount;
+            }
+            let broadcastKey = tabId + '-domSurveyReport';
+            if ( broadcastTimers.has(broadcastKey) === false ) {
+                broadcastTimers.set(broadcastKey, vAPI.setTimeout(
+                    ( ) => { domSurveyFinalReport(tabId); },
+                    53
+                ));
+            }
+        }
+        break;
+
+    case 'inlinescriptFound':
+        if ( µb.logger.enabled && pageStore !== null ) {
+            const fctxt = µb.filteringContext.duplicate();
+            fctxt.fromTabId(tabId)
+                .setType('inline-script')
+                .setURL(request.docURL)
+                .setDocOriginFromURL(request.docURL);
+            if ( pageStore.filterRequest(fctxt) === 0 ) {
+                fctxt.setRealm('network').toLogger();
             }
         }
         break;
