@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,151 +37,134 @@ To create a log of net requests
 
 /******************************************************************************/
 
-var µb = µBlock;
+const µb = µBlock;
 
 /******************************************************************************/
 /******************************************************************************/
 
 // To mitigate memory churning
-var netFilteringCacheJunkyard = [],
-    netFilteringCacheJunkyardMax = 10;
+const netFilteringCacheJunkyard = [];
+const netFilteringCacheJunkyardMax = 10;
 
 /******************************************************************************/
 
-var NetFilteringResultCache = function() {
-    this.boundPruneAsyncCallback = this.pruneAsyncCallback.bind(this);
+const NetFilteringResultCache = function() {
     this.init();
 };
 
-/******************************************************************************/
+NetFilteringResultCache.prototype = {
+    shelfLife: 15000,
 
-NetFilteringResultCache.prototype.shelfLife = 15 * 1000;
+    init: function() {
+        this.blocked = new Map();
+        this.results = new Map();
+        this.hash = 0;
+        this.timer = undefined;
+        return this;
+    },
 
-/******************************************************************************/
+    dispose: function() {
+        this.empty();
+        if ( netFilteringCacheJunkyard.length < netFilteringCacheJunkyardMax ) {
+            netFilteringCacheJunkyard.push(this);
+        }
+        return null;
+    },
+
+    rememberResult: function(fctxt, result) {
+        if ( fctxt.tabId <= 0 ) { return; }
+        if ( this.results.size === 0 ) {
+            this.pruneAsync();
+        }
+        const key = fctxt.getDocHostname() + ' ' + fctxt.type + ' ' + fctxt.url;
+        this.results.set(key, {
+            result: result,
+            logData: fctxt.filter,
+            tstamp: Date.now()
+        });
+        if ( result !== 1 ) { return; }
+        const now = Date.now();
+        this.blocked.set(key, now);
+        this.hash = now;
+    },
+
+    rememberBlock: function(fctxt) {
+        if ( fctxt.tabId <= 0 ) { return; }
+        if ( this.blocked.size === 0 ) {
+            this.pruneAsync();
+        }
+        const now = Date.now();
+        this.blocked.set(
+            fctxt.getDocHostname() + ' ' + fctxt.type + ' ' + fctxt.url,
+            now
+        );
+        this.hash = now;
+    },
+
+    empty: function() {
+        this.blocked.clear();
+        this.results.clear();
+        this.hash = 0;
+        if ( this.timer !== undefined ) {
+            clearTimeout(this.timer);
+            this.timer = undefined;
+        }
+    },
+
+    prune: function() {
+        const obsolete = Date.now() - this.shelfLife;
+        for ( const entry of this.blocked ) {
+            if ( entry[1] <= obsolete ) {
+                this.results.delete(entry[0]);
+                this.blocked.delete(entry[0]);
+            }
+        }
+        for ( const entry of this.results ) {
+            if ( entry[1].tstamp <= obsolete ) {
+                this.results.delete(entry[0]);
+            }
+        }
+        if ( this.blocked.size !== 0 || this.results.size !== 0 ) {
+            this.pruneAsync();
+        }
+    },
+
+    pruneAsync: function() {
+        if ( this.timer !== undefined ) { return; }
+        this.timer = vAPI.setTimeout(
+            ( ) => {
+                this.timer = undefined;
+                this.prune();
+            },
+            this.shelfLife
+        );
+    },
+
+    lookupResult: function(fctxt) {
+        return this.results.get(
+            fctxt.getDocHostname() + ' ' +
+            fctxt.type + ' ' +
+            fctxt.url
+        );
+    },
+
+    lookupAllBlocked: function(hostname) {
+        const result = [];
+        for ( const entry of this.blocked ) {
+            const pos = entry[0].indexOf(' ');
+            if ( entry[0].slice(0, pos) === hostname ) {
+                result[result.length] = entry[0].slice(pos + 1);
+            }
+        }
+        return result;
+    },
+};
 
 NetFilteringResultCache.factory = function() {
-    var entry = netFilteringCacheJunkyard.pop();
-    if ( entry === undefined ) {
-        entry = new NetFilteringResultCache();
-    } else {
-        entry.init();
-    }
-    return entry;
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.init = function() {
-    this.blocked = new Map();
-    this.results = new Map();
-    this.hash = 0;
-    this.timer = null;
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.dispose = function() {
-    this.empty();
-    if ( netFilteringCacheJunkyard.length < netFilteringCacheJunkyardMax ) {
-        netFilteringCacheJunkyard.push(this);
-    }
-    return null;
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.rememberResult = function(context, result, logData) {
-    if ( this.results.size === 0 ) {
-        this.pruneAsync();
-    }
-    var key = context.pageHostname + ' ' + context.requestType + ' ' + context.requestURL;
-    this.results.set(key, {
-        result: result,
-        logData: logData,
-        tstamp: Date.now()
-    });
-    if ( result !== 1 ) { return; }
-    var now = Date.now();
-    this.blocked.set(key, now);
-    this.hash = now;
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.rememberBlock = function(details) {
-    if ( this.blocked.size === 0 ) {
-        this.pruneAsync();
-    }
-    var now = Date.now();
-    this.blocked.set(
-        details.pageHostname + ' ' + details.requestType + ' ' + details.requestURL,
-        now
-    );
-    this.hash = now;
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.empty = function() {
-    this.blocked.clear();
-    this.results.clear();
-    this.hash = 0;
-    if ( this.timer !== null ) {
-        clearTimeout(this.timer);
-        this.timer = null;
-    }
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.pruneAsync = function() {
-    if ( this.timer === null ) {
-        this.timer = vAPI.setTimeout(this.boundPruneAsyncCallback, this.shelfLife * 2);
-    }
-};
-
-NetFilteringResultCache.prototype.pruneAsyncCallback = function() {
-    this.timer = null;
-    var obsolete = Date.now() - this.shelfLife,
-        entry;
-    for ( entry of this.blocked ) {
-        if ( entry[1] <= obsolete ) {
-            this.results.delete(entry[0]);
-            this.blocked.delete(entry[0]);
-        }
-    }
-    for ( entry of this.results ) {
-        if ( entry[1].tstamp <= obsolete ) {
-            this.results.delete(entry[0]);
-        }
-    }
-    if ( this.blocked.size !== 0 || this.results.size !== 0 ) {
-        this.pruneAsync();
-    }
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.lookupResult = function(context) {
-    return this.results.get(
-        context.pageHostname + ' ' +
-        context.requestType + ' ' +
-        context.requestURL
-    );
-};
-
-/******************************************************************************/
-
-NetFilteringResultCache.prototype.lookupAllBlocked = function(hostname) {
-    var result = [],
-        pos;
-    for ( var entry of this.blocked ) {
-        pos = entry[0].indexOf(' ');
-        if ( entry[0].slice(0, pos) === hostname ) {
-            result[result.length] = entry[0].slice(pos + 1);
-        }
-    }
-    return result;
+    const entry = netFilteringCacheJunkyard.pop();
+    return entry !== undefined
+        ? entry.init()
+        : new NetFilteringResultCache();
 };
 
 /******************************************************************************/
@@ -193,54 +176,51 @@ NetFilteringResultCache.prototype.lookupAllBlocked = function(hostname) {
 // refactoring.
 
 // To mitigate memory churning
-var frameStoreJunkyard = [];
-var frameStoreJunkyardMax = 50;
+const frameStoreJunkyard = [];
+const frameStoreJunkyardMax = 50;
 
 /******************************************************************************/
 
-var FrameStore = function(frameURL) {
+const FrameStore = function(frameURL) {
     this.init(frameURL);
 };
 
-/******************************************************************************/
+FrameStore.prototype = {
+    init: function(frameURL) {
+        const µburi = µb.URI;
+        this.pageHostname = µburi.hostnameFromURI(frameURL);
+        this.pageDomain =
+            µburi.domainFromHostname(this.pageHostname) || this.pageHostname;
+        return this;
+    },
+
+    dispose: function() {
+        this.pageHostname = this.pageDomain = '';
+        if ( frameStoreJunkyard.length < frameStoreJunkyardMax ) {
+            frameStoreJunkyard.push(this);
+        }
+        return null;
+    },
+};
 
 FrameStore.factory = function(frameURL) {
-    var entry = frameStoreJunkyard.pop();
+    const entry = frameStoreJunkyard.pop();
     if ( entry === undefined ) {
         return new FrameStore(frameURL);
     }
     return entry.init(frameURL);
 };
 
-/******************************************************************************/
-
-FrameStore.prototype.init = function(frameURL) {
-    var µburi = µb.URI;
-    this.pageHostname = µburi.hostnameFromURI(frameURL);
-    this.pageDomain = µburi.domainFromHostname(this.pageHostname) || this.pageHostname;
-    return this;
-};
-
-/******************************************************************************/
-
-FrameStore.prototype.dispose = function() {
-    this.pageHostname = this.pageDomain = '';
-    if ( frameStoreJunkyard.length < frameStoreJunkyardMax ) {
-        frameStoreJunkyard.push(this);
-    }
-    return null;
-};
-
 /******************************************************************************/
 /******************************************************************************/
 
 // To mitigate memory churning
-var pageStoreJunkyard = [];
-var pageStoreJunkyardMax = 10;
+const pageStoreJunkyard = [];
+const pageStoreJunkyardMax = 10;
 
 /******************************************************************************/
 
-var PageStore = function(tabId, context) {
+const PageStore = function(tabId, context) {
     this.init(tabId, context);
     this.journal = [];
     this.journalTimer = null;
@@ -251,7 +231,7 @@ var PageStore = function(tabId, context) {
 /******************************************************************************/
 
 PageStore.factory = function(tabId, context) {
-    var entry = pageStoreJunkyard.pop();
+    let entry = pageStoreJunkyard.pop();
     if ( entry === undefined ) {
         entry = new PageStore(tabId, context);
     } else {
@@ -267,7 +247,7 @@ PageStore.factory = function(tabId, context) {
 //   logger.
 
 PageStore.prototype.init = function(tabId, context) {
-    var tabContext = µb.tabContextManager.mustLookup(tabId);
+    const tabContext = µb.tabContextManager.mustLookup(tabId);
     this.tabId = tabId;
 
     // If we are navigating from-to same site, remember whether large
@@ -290,55 +270,63 @@ PageStore.prototype.init = function(tabId, context) {
     this.perLoadAllowedRequestCount = 0;
     this.hiddenElementCount = ''; // Empty string means "unknown"
     this.remoteFontCount = 0;
+    this.scriptCount = 0;
     this.popupBlockedCount = 0;
     this.largeMediaCount = 0;
     this.largeMediaTimer = null;
     this.netFilteringCache = NetFilteringResultCache.factory();
     this.internalRedirectionCount = 0;
 
-    this.noCosmeticFiltering = µb.hnSwitches.evaluateZ(
+    // The current filtering context is cloned because:
+    // - We may be called with or without the current context having been
+    //   initialized.
+    // - If it has been initialized, we do not want to change the state
+    //   of the current context.
+    const fctxt = µb.logger.enabled
+        ? µBlock.filteringContext
+                .duplicate()
+                .fromTabId(tabId)
+                .setURL(tabContext.rawURL)
+        : undefined;
+
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/314
+    const masterSwitch = tabContext.getNetFilteringSwitch();
+
+    this.noCosmeticFiltering = µb.sessionSwitches.evaluateZ(
         'no-cosmetic-filtering',
         tabContext.rootHostname
     ) === true;
     if (
+        masterSwitch &&
         this.noCosmeticFiltering &&
-        µb.logger.isEnabled() &&
+        µb.logger.enabled &&
         context === 'tabCommitted'
     ) {
-        µb.logger.writeOne(
-            tabId,
-            'cosmetic',
-            µb.hnSwitches.toLogData(),
-            'dom',
-            tabContext.rawURL,
-            this.tabHostname,
-            this.tabHostname
-        );
+        fctxt.setRealm('cosmetic')
+             .setType('dom')
+             .setFilter(µb.sessionSwitches.toLogData())
+             .toLogger();
     }
 
     // Support `generichide` filter option.
-    this.noGenericCosmeticFiltering = this.noCosmeticFiltering;
+    this.noGenericCosmeticFiltering = masterSwitch !== true;
     if ( this.noGenericCosmeticFiltering !== true ) {
-        var result = µb.staticNetFilteringEngine.matchStringExactType(
-            this.createContextFromPage(),
-            tabContext.normalURL,
-            'generichide'
-        );
-        this.noGenericCosmeticFiltering = result === 2;
-        if (
-            result !== 0 &&
-            µb.logger.isEnabled() &&
-            context === 'tabCommitted'
-        ) {
-            µb.logger.writeOne(
-                tabId,
-                'net',
-                µb.staticNetFilteringEngine.toLogData(),
-                'generichide',
-                tabContext.rawURL,
-                this.tabHostname,
-                this.tabHostname
+        this.noGenericCosmeticFiltering = this.noCosmeticFiltering;
+        if ( this.noGenericCosmeticFiltering !== true ) {
+            let result = µb.staticNetFilteringEngine.matchStringGenericHide(
+                tabContext.normalURL
             );
+            this.noGenericCosmeticFiltering = result === 2;
+            if (
+                result !== 0 &&
+                µb.logger.enabled &&
+                context === 'tabCommitted'
+            ) {
+                fctxt.setRealm('network')
+                     .setType('generichide')
+                     .setFilter(µb.staticNetFilteringEngine.toLogData())
+                     .toLogger();
+            }
         }
     }
 
@@ -351,7 +339,7 @@ PageStore.prototype.reuse = function(context) {
     // When force refreshing a page, the page store data needs to be reset.
 
     // If the hostname changes, we can't merely just update the context.
-    var tabContext = µb.tabContextManager.mustLookup(this.tabId);
+    const tabContext = µb.tabContextManager.mustLookup(this.tabId);
     if ( tabContext.rootHostname !== this.tabHostname ) {
         context = '';
     }
@@ -410,22 +398,18 @@ PageStore.prototype.dispose = function() {
 /******************************************************************************/
 
 PageStore.prototype.disposeFrameStores = function() {
-    for ( var frameStore of this.frames.values() ) {
+    for ( const frameStore of this.frames.values() ) {
         frameStore.dispose();
     }
     this.frames.clear();
 };
 
-/******************************************************************************/
-
 PageStore.prototype.getFrame = function(frameId) {
     return this.frames.get(frameId) || null;
 };
 
-/******************************************************************************/
-
 PageStore.prototype.setFrame = function(frameId, frameURL) {
-    var frameStore = this.frames.get(frameId);
+    const frameStore = this.frames.get(frameId);
     if ( frameStore !== undefined ) {
         frameStore.init(frameURL);
     } else {
@@ -435,54 +419,19 @@ PageStore.prototype.setFrame = function(frameId, frameURL) {
 
 /******************************************************************************/
 
-PageStore.prototype.createContextFromPage = function() {
-    var context = µb.tabContextManager.createContext(this.tabId);
-    context.pageHostname = context.rootHostname;
-    context.pageDomain = context.rootDomain;
-    return context;
-};
-
-PageStore.prototype.createContextFromFrameId = function(frameId) {
-    var context = µb.tabContextManager.createContext(this.tabId);
-    var frameStore = this.frames.get(frameId);
-    if ( frameStore !== undefined ) {
-        context.pageHostname = frameStore.pageHostname;
-        context.pageDomain = frameStore.pageDomain;
-    } else {
-        context.pageHostname = context.rootHostname;
-        context.pageDomain = context.rootDomain;
-    }
-    return context;
-};
-
-PageStore.prototype.createContextFromFrameHostname = function(frameHostname) {
-    var context = µb.tabContextManager.createContext(this.tabId);
-    context.pageHostname = frameHostname;
-    context.pageDomain = µb.URI.domainFromHostname(frameHostname) || frameHostname;
-    return context;
-};
-
-/******************************************************************************/
-
 PageStore.prototype.getNetFilteringSwitch = function() {
     return µb.tabContextManager.mustLookup(this.tabId).getNetFilteringSwitch();
 };
 
-/******************************************************************************/
-
 PageStore.prototype.getSpecificCosmeticFilteringSwitch = function() {
     return this.noCosmeticFiltering !== true;
 };
 
-/******************************************************************************/
-
 PageStore.prototype.getGenericCosmeticFilteringSwitch = function() {
     return this.noGenericCosmeticFiltering !== true &&
            this.noCosmeticFiltering !== true;
 };
 
-/******************************************************************************/
-
 PageStore.prototype.toggleNetFilteringSwitch = function(url, scope, state) {
     µb.toggleNetFilteringSwitch(url, scope, state);
     this.netFilteringCache.empty();
@@ -519,7 +468,10 @@ PageStore.prototype.journalAddRequest = function(hostname, result) {
         result === 1 ? 0x00000001 : 0x00010000
     );
     if ( this.journalTimer === null ) {
-        this.journalTimer = vAPI.setTimeout(this.journalProcess.bind(this, true), 1000);
+        this.journalTimer = vAPI.setTimeout(
+            ( ) => { this.journalProcess(true); },
+            µb.hiddenSettings.requestJournalProcessPeriod
+        );
     }
 };
 
@@ -541,7 +493,10 @@ PageStore.prototype.journalAddRootFrame = function(type, url) {
     if ( this.journalTimer !== null ) {
         clearTimeout(this.journalTimer);
     }
-    this.journalTimer = vAPI.setTimeout(this.journalProcess.bind(this, true), 1000);
+    this.journalTimer = vAPI.setTimeout(
+        ( ) => { this.journalProcess(true); },
+        µb.hiddenSettings.requestJournalProcessPeriod
+    );
 };
 
 PageStore.prototype.journalProcess = function(fromTimer) {
@@ -550,22 +505,20 @@ PageStore.prototype.journalProcess = function(fromTimer) {
     }
     this.journalTimer = null;
 
-    var journal = this.journal,
-        i, n = journal.length,
-        hostname, count, hostnameCounts,
-        aggregateCounts = 0,
-        now = Date.now(),
-        pivot = this.journalLastCommitted || 0;
+    const journal = this.journal;
+    const now = Date.now();
+    let aggregateCounts = 0;
+    let pivot = this.journalLastCommitted || 0;
 
     // Everything after pivot originates from current page.
-    for ( i = pivot; i < n; i += 2 ) {
-        hostname = journal[i];
-        hostnameCounts = this.hostnameToCountMap.get(hostname);
+    for ( let i = pivot; i < journal.length; i += 2 ) {
+        const hostname = journal[i];
+        let hostnameCounts = this.hostnameToCountMap.get(hostname);
         if ( hostnameCounts === undefined ) {
             hostnameCounts = 0;
             this.contentLastModified = now;
         }
-        count = journal[i+1];
+        let count = journal[i+1];
         this.hostnameToCountMap.set(hostname, hostnameCounts + count);
         aggregateCounts += count;
     }
@@ -581,7 +534,7 @@ PageStore.prototype.journalProcess = function(fromTimer) {
 
     // Everything before pivot does not originate from current page -- we still
     // need to bump global blocked/allowed counts.
-    for ( i = 0; i < pivot; i += 2 ) {
+    for ( let i = 0; i < pivot; i += 2 ) {
         aggregateCounts += journal[i+1];
     }
     if ( aggregateCounts !== 0 ) {
@@ -594,97 +547,101 @@ PageStore.prototype.journalProcess = function(fromTimer) {
 
 /******************************************************************************/
 
-PageStore.prototype.filterRequest = function(context) {
-    this.logData = undefined;
+PageStore.prototype.filterRequest = function(fctxt) {
+    fctxt.filter = undefined;
 
     if ( this.getNetFilteringSwitch() === false ) {
         return 0;
     }
 
-    var requestType = context.requestType;
+    const requestType = fctxt.type;
 
-    if ( requestType === 'csp_report' && this.filterCSPReport(context) === 1 ) {
+    if ( requestType === 'csp_report' && this.filterCSPReport(fctxt) === 1 ) {
         return 1;
     }
 
-    if ( requestType.endsWith('font') && this.filterFont(context) === 1 ) {
+    if ( requestType.endsWith('font') && this.filterFont(fctxt) === 1 ) {
         return 1;
     }
 
-    var cacheableResult = this.cacheableResults[requestType] === true;
+    if (
+        requestType === 'script' &&
+        this.filterScripting(fctxt, true) === 1
+    ) {
+        return 1;
+    }
+
+    const cacheableResult = this.cacheableResults.has(requestType);
 
     if ( cacheableResult ) {
-        var entry = this.netFilteringCache.lookupResult(context);
+        const entry = this.netFilteringCache.lookupResult(fctxt);
         if ( entry !== undefined ) {
-            this.logData = entry.logData;
+            fctxt.filter = entry.logData;
             return entry.result;
         }
     }
 
     // Dynamic URL filtering.
-    var result = µb.sessionURLFiltering.evaluateZ(context.rootHostname, context.requestURL, requestType);
-    if ( result !== 0 && µb.logger.isEnabled() ) {
-        this.logData = µb.sessionURLFiltering.toLogData();
+    let result = µb.sessionURLFiltering.evaluateZ(
+        fctxt.getTabHostname(),
+        fctxt.url,
+        requestType
+    );
+    if ( result !== 0 && µb.logger.enabled ) {
+        fctxt.filter = µb.sessionURLFiltering.toLogData();
     }
 
     // Dynamic hostname/type filtering.
     if ( result === 0 && µb.userSettings.advancedUserEnabled ) {
-        result = µb.sessionFirewall.evaluateCellZY(context.rootHostname, context.requestHostname, requestType);
-        if ( result !== 0 && result !== 3 && µb.logger.isEnabled() ) {
-            this.logData = µb.sessionFirewall.toLogData();
+        result = µb.sessionFirewall.evaluateCellZY(
+            fctxt.getTabHostname(),
+            fctxt.getHostname(),
+            requestType
+        );
+        if ( result !== 0 && result !== 3 && µb.logger.enabled ) {
+            fctxt.filter = µb.sessionFirewall.toLogData();
         }
     }
 
     // Static filtering has lowest precedence.
     if ( result === 0 || result === 3 ) {
-        result = µb.staticNetFilteringEngine.matchString(context);
-        if ( result !== 0 && µb.logger.isEnabled() ) {
-            this.logData = µb.staticNetFilteringEngine.toLogData();
+        result = µb.staticNetFilteringEngine.matchString(fctxt);
+        if ( result !== 0 && µb.logger.enabled ) {
+            fctxt.filter = µb.staticNetFilteringEngine.toLogData();
         }
     }
 
     if ( cacheableResult ) {
-        this.netFilteringCache.rememberResult(context, result, this.logData);
-    } else if ( result === 1 && this.collapsibleResources[requestType] === true ) {
-        this.netFilteringCache.rememberBlock(context, true);
+        this.netFilteringCache.rememberResult(fctxt, result);
+    } else if ( result === 1 && this.collapsibleResources.has(requestType) ) {
+        this.netFilteringCache.rememberBlock(fctxt, true);
     }
 
     return result;
 };
 
-PageStore.prototype.cacheableResults = {
-    sub_frame: true
-};
+PageStore.prototype.cacheableResults = new Set([
+    'sub_frame',
+]);
 
-PageStore.prototype.collapsibleResources = {
-    image: true,
-    media: true,
-    object: true,
-    sub_frame: true
-};
+PageStore.prototype.collapsibleResources = new Set([
+    'image',
+    'media',
+    'object',
+    'sub_frame',
+]);
 
 /******************************************************************************/
 
-PageStore.prototype.filterCSPReport = function(context) {
-    if ( µb.hnSwitches.evaluateZ('no-csp-reports', context.requestHostname) ) {
-        if ( µb.logger.isEnabled() ) {
-            this.logData = µb.hnSwitches.toLogData();
-        }
-        return 1;
-    }
-    // https://github.com/gorhill/uBlock/issues/3140
-    //   Special handling of CSP reports if and only if these can't be filtered
-    //   natively.
+PageStore.prototype.filterCSPReport = function(fctxt) {
     if (
-        vAPI.net.nativeCSPReportFiltering !== true &&
-        this.internalRedirectionCount !== 0
+        µb.sessionSwitches.evaluateZ(
+            'no-csp-reports',
+            fctxt.getHostname()
+        )
     ) {
-        if ( µb.logger.isEnabled() ) {
-            this.logData = {
-                result: 1,
-                source: 'global',
-                raw: 'no-spurious-csp-report'
-            };
+        if ( µb.logger.enabled ) {
+            fctxt.filter = µb.sessionSwitches.toLogData();
         }
         return 1;
     }
@@ -693,13 +650,18 @@ PageStore.prototype.filterCSPReport = function(context) {
 
 /******************************************************************************/
 
-PageStore.prototype.filterFont = function(context) {
-    if ( context.requestType === 'font' ) {
+PageStore.prototype.filterFont = function(fctxt) {
+    if ( fctxt.type === 'font' ) {
         this.remoteFontCount += 1;
     }
-    if ( µb.hnSwitches.evaluateZ('no-remote-fonts', context.rootHostname) !== false ) {
-        if ( µb.logger.isEnabled() ) {
-            this.logData = µb.hnSwitches.toLogData();
+    if (
+        µb.sessionSwitches.evaluateZ(
+            'no-remote-fonts',
+            fctxt.getTabHostname()
+        ) !== false
+    ) {
+        if ( µb.logger.enabled ) {
+            fctxt.filter = µb.sessionSwitches.toLogData();
         }
         return 1;
     }
@@ -708,15 +670,42 @@ PageStore.prototype.filterFont = function(context) {
 
 /******************************************************************************/
 
+PageStore.prototype.filterScripting = function(fctxt, netFiltering) {
+    fctxt.filter = undefined;
+    if ( netFiltering === undefined ) {
+        netFiltering = this.getNetFilteringSwitch();
+    }
+    if (
+        netFiltering === false ||
+        µb.sessionSwitches.evaluateZ(
+            'no-scripting',
+            fctxt.getTabHostname()
+        ) === false
+    ) {
+        return 0;
+    }
+    if ( µb.logger.enabled ) {
+        fctxt.filter = µb.sessionSwitches.toLogData();
+    }
+    return 1;
+};
+
+/******************************************************************************/
+
 // The caller is responsible to check whether filtering is enabled or not.
 
-PageStore.prototype.filterLargeMediaElement = function(size) {
-    this.logData = undefined;
+PageStore.prototype.filterLargeMediaElement = function(fctxt, size) {
+    fctxt.filter = undefined;
 
     if ( Date.now() < this.allowLargeMediaElementsUntil ) {
         return 0;
     }
-    if ( µb.hnSwitches.evaluateZ('no-large-media', this.tabHostname) !== true ) {
+    if (
+        µb.sessionSwitches.evaluateZ(
+            'no-large-media',
+            fctxt.getTabHostname()
+        ) !== true
+    ) {
         return 0;
     }
     if ( (size >>> 10) < µb.userSettings.largeMediaSize ) {
@@ -731,8 +720,8 @@ PageStore.prototype.filterLargeMediaElement = function(size) {
         );
     }
 
-    if ( µb.logger.isEnabled() ) {
-        this.logData = µb.hnSwitches.toLogData();
+    if ( µb.logger.enabled ) {
+        fctxt.filter = µb.sessionSwitches.toLogData();
     }
 
     return 1;
@@ -743,26 +732,27 @@ PageStore.prototype.filterLargeMediaElement = function(size) {
 /******************************************************************************/
 
 PageStore.prototype.getBlockedResources = function(request, response) {
-    var µburi = µb.URI,
-        normalURL = µb.normalizePageURL(this.tabId, request.frameURL),
-        frameHostname = µburi.hostnameFromURI(normalURL),
-        resources = request.resources;
+    const normalURL = µb.normalizePageURL(this.tabId, request.frameURL);
+    const resources = request.resources;
+    const fctxt = µBlock.filteringContext;
+    fctxt.fromTabId(this.tabId)
+         .setDocOriginFromURL(normalURL);
     // Force some resources to go through the filtering engine in order to
     // populate the blocked-resources cache. This is required because for
     // some resources it's not possible to detect whether they were blocked
     // content script-side (i.e. `iframes` -- unlike `img`).
     if ( Array.isArray(resources) && resources.length !== 0 ) {
-        var context = this.createContextFromFrameHostname(frameHostname);
-        for ( var resource of resources ) {
-            context.requestType = resource.type;
-            context.requestHostname = µburi.hostnameFromURI(resource.url);
-            context.requestURL = resource.url;
-            this.filterRequest(context);
+        for ( const resource of resources ) {
+            this.filterRequest(
+                fctxt.setType(resource.type)
+                     .setURL(resource.url)
+            );
         }
     }
     if ( this.netFilteringCache.hash === response.hash ) { return; }
     response.hash = this.netFilteringCache.hash;
-    response.blockedResources = this.netFilteringCache.lookupAllBlocked(frameHostname);
+    response.blockedResources =
+        this.netFilteringCache.lookupAllBlocked(fctxt.getDocHostname());
 };
 
 /******************************************************************************/
