@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 
 /******************************************************************************/
 
-var popupFontSize = vAPI.localStorage.getItem('popupFontSize');
+let popupFontSize = vAPI.localStorage.getItem('popupFontSize');
 if ( typeof popupFontSize === 'string' && popupFontSize !== 'unset' ) {
     document.body.style.setProperty('font-size', popupFontSize);
 }
@@ -58,24 +58,24 @@ if ( dfPaneVisibleStored ) {
 
 /******************************************************************************/
 
-var messaging = vAPI.messaging;
-var popupData = {};
-var dfPaneBuilt = false;
-var reIP = /^\d+(?:\.\d+){1,3}$/;
-var scopeToSrcHostnameMap = {
+const messaging = vAPI.messaging;
+const reIP = /^\d+(?:\.\d+){1,3}$/;
+const scopeToSrcHostnameMap = {
     '/': '*',
     '.': ''
 };
-var dfHotspots = null;
-var hostnameToSortableTokenMap = new Map();
-var allDomains = {};
-var allDomainCount = 0;
-var allHostnameRows = [];
-var touchedDomainCount = 0;
-var rowsToRecycle = uDom();
-var cachedPopupHash = '';
-var statsStr = vAPI.i18n('popupBlockedStats');
-var domainsHitStr = vAPI.i18n('popupHitDomainCount');
+const hostnameToSortableTokenMap = new Map();
+const statsStr = vAPI.i18n('popupBlockedStats');
+const domainsHitStr = vAPI.i18n('popupHitDomainCount');
+
+let popupData = {};
+let dfPaneBuilt = false;
+let dfHotspots = null;
+let allDomains = {};
+let allDomainCount = 0;
+let allHostnameRows = [];
+let touchedDomainCount = 0;
+let cachedPopupHash = '';
 
 // https://github.com/gorhill/uBlock/issues/2550
 // Solution inspired from
@@ -85,8 +85,8 @@ var domainsHitStr = vAPI.i18n('popupHitDomainCount');
 // - http://unicode.org/cldr/utility/list-unicodeset.jsp?a=%5B%D0%B0%D1%81%D4%81%D0%B5%D2%BB%D1%96%D1%98%D3%8F%D0%BE%D1%80%D4%9B%D1%95%D4%9D%D1%85%D1%83%D1%8A%D0%AC%D2%BD%D0%BF%D0%B3%D1%B5%D1%A1%5D&g=gc&i=
 // Linked from:
 // - https://www.chromium.org/developers/design-documents/idn-in-google-chrome
-var reCyrillicNonAmbiguous = /[\u0400-\u042b\u042d-\u042f\u0431\u0432\u0434\u0436-\u043d\u0442\u0444\u0446-\u0449\u044b-\u0454\u0457\u0459-\u0460\u0462-\u0474\u0476-\u04ba\u04bc\u04be-\u04ce\u04d0-\u0500\u0502-\u051a\u051c\u051e-\u052f]/;
-var reCyrillicAmbiguous = /[\u042c\u0430\u0433\u0435\u043e\u043f\u0440\u0441\u0443\u0445\u044a\u0455\u0456\u0458\u0461\u0475\u04bb\u04bd\u04cf\u0501\u051b\u051d]/;
+const reCyrillicNonAmbiguous = /[\u0400-\u042b\u042d-\u042f\u0431\u0432\u0434\u0436-\u043d\u0442\u0444\u0446-\u0449\u044b-\u0454\u0457\u0459-\u0460\u0462-\u0474\u0476-\u04ba\u04bc\u04be-\u04ce\u04d0-\u0500\u0502-\u051a\u051c\u051e-\u052f]/;
+const reCyrillicAmbiguous = /[\u042c\u0430\u0433\u0435\u043e\u043f\u0440\u0441\u0443\u0445\u044a\u0455\u0456\u0458\u0461\u0475\u04bb\u04bd\u04cf\u0501\u051b\u051d]/;
 
 /******************************************************************************/
 
@@ -94,21 +94,21 @@ var reCyrillicAmbiguous = /[\u042c\u0430\u0433\u0435\u043e\u043f\u0440\u0441\u04
 // - Its vertical position depends on the height of the popup title bar
 // - Its horizontal position depends on whether there is a vertical scrollbar.
 
-var positionRulesetTools = function() {
-    var vpos = document.getElementById('appinfo')
-                       .getBoundingClientRect()
-                       .bottom + window.scrollY + 3;
-    var hpos = document.getElementById('firewallContainer')
-                       .getBoundingClientRect()
-                       .left + window.scrollX + 3;
-    var style = document.getElementById('rulesetTools').style;
+const positionRulesetTools = function() {
+    const vpos = document.getElementById('appinfo')
+                         .getBoundingClientRect()
+                         .bottom + window.scrollY + 3;
+    const hpos = document.getElementById('firewallContainer')
+                         .getBoundingClientRect()
+                         .left + window.scrollX + 3;
+    const style = document.getElementById('rulesetTools').style;
     style.setProperty('top', (vpos >>> 0) + 'px');
     style.setProperty('left', (hpos >>> 0) + 'px');
 };
 
 /******************************************************************************/
 
-var cachePopupData = function(data) {
+const cachePopupData = function(data) {
     popupData = {};
     scopeToSrcHostnameMap['.'] = '';
     hostnameToSortableTokenMap.clear();
@@ -118,11 +118,11 @@ var cachePopupData = function(data) {
     }
     popupData = data;
     scopeToSrcHostnameMap['.'] = popupData.pageHostname || '';
-    let hostnameDict = popupData.hostnameDict;
+    const hostnameDict = popupData.hostnameDict;
     if ( typeof hostnameDict !== 'object' ) {
         return popupData;
     }
-    for ( let hostname in hostnameDict ) {
+    for ( const hostname in hostnameDict ) {
         if ( hostnameDict.hasOwnProperty(hostname) === false ) { continue; }
         let domain = hostnameDict[hostname].domain;
         let prefix = hostname.slice(0, 0 - domain.length - 1);
@@ -141,28 +141,33 @@ var cachePopupData = function(data) {
 
 /******************************************************************************/
 
-var hashFromPopupData = function(reset) {
+const hashFromPopupData = function(reset) {
     // It makes no sense to offer to refresh the behind-the-scene scope
     if ( popupData.pageHostname === 'behind-the-scene' ) {
         uDom('body').toggleClass('dirty', false);
         return;
     }
 
-    var hasher = [],
-        rules = popupData.firewallRules;
-    for ( var key in rules ) {
-        var rule = rules[key];
-        if ( rule !== null ) {
-            hasher.push(rule.src + ' ' + rule.des + ' ' + rule.type + ' ' + rule.action);
-        }
+    const hasher = [];
+    const rules = popupData.firewallRules;
+    for ( const key in rules ) {
+        const rule = rules[key];
+        if ( rule === null ) { continue; }
+        hasher.push(
+            rule.src + ' ' +
+            rule.des + ' ' +
+            rule.type + ' ' +
+            rule.action
+        );
     }
     hasher.sort();
     hasher.push(uDom('body').hasClass('off'));
     hasher.push(uDom.nodeFromId('no-large-media').classList.contains('on'));
     hasher.push(uDom.nodeFromId('no-cosmetic-filtering').classList.contains('on'));
     hasher.push(uDom.nodeFromId('no-remote-fonts').classList.contains('on'));
+    hasher.push(uDom.nodeFromId('no-scripting').classList.contains('on'));
 
-    var hash = hasher.join('');
+    const hash = hasher.join('');
     if ( reset ) {
         cachedPopupHash = hash;
     }
@@ -171,23 +176,23 @@ var hashFromPopupData = function(reset) {
 
 /******************************************************************************/
 
-var formatNumber = function(count) {
+const formatNumber = function(count) {
     return typeof count === 'number' ? count.toLocaleString() : '';
 };
 
 /******************************************************************************/
 
-var rulekeyCompare = function(a, b) {
-    var ha = a.slice(2, a.indexOf(' ', 2));
+const rulekeyCompare = function(a, b) {
+    let ha = a.slice(2, a.indexOf(' ', 2));
     if ( !reIP.test(ha) ) {
         ha = hostnameToSortableTokenMap.get(ha) || ' ';
     }
-    var hb = b.slice(2, b.indexOf(' ', 2));
+    let hb = b.slice(2, b.indexOf(' ', 2));
     if ( !reIP.test(hb) ) {
         hb = hostnameToSortableTokenMap.get(hb) || ' ';
     }
-    var ca = ha.charCodeAt(0),
-        cb = hb.charCodeAt(0);
+    const ca = ha.charCodeAt(0);
+    const cb = hb.charCodeAt(0);
     if ( ca !== cb ) {
         return ca - cb;
     }
@@ -196,76 +201,46 @@ var rulekeyCompare = function(a, b) {
 
 /******************************************************************************/
 
-var addFirewallRow = function(des) {
-    var row = rowsToRecycle.pop();
-    if ( row.length === 0 ) {
-        row = uDom('#templates > div:nth-of-type(1)').clone();
-    }
-
-    row.descendants('[data-des]').attr('data-des', des);
-
-    var hnDetails = popupData.hostnameDict[des] || {},
-        isDomain = des === hnDetails.domain,
-        prettyDomainName = punycode.toUnicode(des),
-        isPunycoded = prettyDomainName !== des;
-    var span = row.nodeAt(0).querySelector('span:first-of-type');
-    span.classList.toggle(
-        'isIDN',
-        isPunycoded && reCyrillicAmbiguous.test(prettyDomainName) === true &&
-                       reCyrillicNonAmbiguous.test(prettyDomainName) === false
+const updateFirewallCell = function(scope, des, type, rule) {
+    const row = document.querySelector(
+        `#firewallContainer div[data-des="${des}"][data-type="${type}"]`
     );
-    span.querySelector('span').textContent = prettyDomainName;
-    span.title = isDomain && isPunycoded ? des : '';
+    if ( row === null ) { return; }
 
-    row.toggleClass('isDomain', isDomain)
-       .toggleClass('isSubDomain', !isDomain)
-       .toggleClass('allowed', hnDetails.allowCount !== 0)
-       .toggleClass('blocked', hnDetails.blockCount !== 0)
-       .toggleClass('totalAllowed', hnDetails.totalAllowCount !== 0)
-       .toggleClass('totalBlocked', hnDetails.totalBlockCount !== 0);
+    const cells = row.querySelectorAll(`:scope > span[data-src="${scope}"]`);
+    if ( cells.length === 0 ) { return; }
 
-    row.appendTo('#firewallContainer');
-
-    return row;
-};
-
-/******************************************************************************/
-
-var updateFirewallCell = function(scope, des, type, rule) {
-    var selector = '#firewallContainer span[data-src="' + scope + '"][data-des="' + des + '"][data-type="' + type + '"]';
-    var cells = uDom(selector);
-    if ( cells.length === 0 ) {
-        return;
-    }
-
-    cells.removeClass();
     if ( rule !== null ) {
-        cells.toggleClass(rule.action + 'Rule', true);
+        cells.forEach(el => { el.setAttribute('class', rule.action + 'Rule'); });
+    } else {
+        cells.forEach(el => { el.removeAttribute('class'); });
     }
 
     // Use dark shade visual cue if the rule is specific to the cell.
-    var ownRule = false;
-    if ( rule !== null ) {
-        ownRule = (rule.des !== '*' || rule.type === type) &&
-                  (rule.des === des) &&
-                  (rule.src === scopeToSrcHostnameMap[scope]);
+    if (
+        (rule !== null) &&
+        (rule.des !== '*' || rule.type === type) &&
+        (rule.des === des) &&
+        (rule.src === scopeToSrcHostnameMap[scope])
+        
+    ) {
+        cells.forEach(el => { el.classList.add('ownRule'); });
     }
-    cells.toggleClass('ownRule', ownRule);
 
-    if ( scope !== '.' || des === '*' ) {
-        return;
-    }
+    if ( scope !== '.' || des === '*' ) { return; }
 
     // Remember this may be a cell from a reused row, we need to clear text
     // content if we can't compute request counts.
     if ( popupData.hostnameDict.hasOwnProperty(des) === false ) {
-        cells.removeAttr('data-acount');
-        cells.removeAttr('data-acount');
+        cells.forEach(el => {
+            el.removeAttribute('data-acount');
+            el.removeAttribute('data-bcount');
+        });
         return;
     }
 
-    var hnDetails = popupData.hostnameDict[des],
-        cell = cells.nodeAt(0);
+    const hnDetails = popupData.hostnameDict[des];
+    let cell = cells[0];
     if ( hnDetails.allowCount !== 0 ) {
         cell.setAttribute('data-acount', Math.min(Math.ceil(Math.log(hnDetails.allowCount + 1) / Math.LN10), 3));
     } else {
@@ -281,7 +256,7 @@ var updateFirewallCell = function(scope, des, type, rule) {
         return;
     }
 
-    cell = cells.nodeAt(1);
+    cell = cells[1];
     if ( hnDetails.totalAllowCount !== 0 ) {
         cell.setAttribute('data-acount', Math.min(Math.ceil(Math.log(hnDetails.totalAllowCount + 1) / Math.LN10), 3));
     } else {
@@ -296,12 +271,10 @@ var updateFirewallCell = function(scope, des, type, rule) {
 
 /******************************************************************************/
 
-var updateAllFirewallCells = function() {
-    var rules = popupData.firewallRules;
-    for ( var key in rules ) {
-        if ( rules.hasOwnProperty(key) === false ) {
-            continue;
-        }
+const updateAllFirewallCells = function() {
+    const rules = popupData.firewallRules;
+    for ( const key in rules ) {
+        if ( rules.hasOwnProperty(key) === false ) { continue; }
         updateFirewallCell(
             key.charAt(0),
             key.slice(2, key.indexOf(' ', 2)),
@@ -310,7 +283,7 @@ var updateAllFirewallCells = function() {
         );
     }
 
-    var dirty = popupData.matrixIsDirty === true;
+    const dirty = popupData.matrixIsDirty === true;
     if ( dirty ) {
         positionRulesetTools();
     }
@@ -319,7 +292,7 @@ var updateAllFirewallCells = function() {
 
 /******************************************************************************/
 
-var buildAllFirewallRows = function() {
+const buildAllFirewallRows = function() {
     // Do this before removing the rows
     if ( dfHotspots === null ) {
         dfHotspots = uDom('#actionSelector')
@@ -328,13 +301,57 @@ var buildAllFirewallRows = function() {
     }
     dfHotspots.detach();
 
-    // Remove and reuse all rows: the order may have changed, we can't just
-    // reuse them in-place.
-    rowsToRecycle = uDom('#firewallContainer > div:nth-of-type(7) ~ div').detach();
+    // Update incrementally: reuse existing rows if possible.
+    let rowContainer = document.getElementById('firewallContainer');
+    let toAppend = document.createDocumentFragment();
+    let rowTemplate = document.querySelector('#templates > div:nth-of-type(1)');
+    let row = rowContainer.querySelector('div:nth-of-type(7) + div');
 
-    var n = allHostnameRows.length;
-    for ( var i = 0; i < n; i++ ) {
-        addFirewallRow(allHostnameRows[i]);
+    for ( const des of allHostnameRows ) {
+        if ( row === null ) {
+            row = rowTemplate.cloneNode(true);
+            toAppend.appendChild(row);
+        }
+
+        row.setAttribute('data-des', des);
+
+        const hnDetails = popupData.hostnameDict[des] || {};
+        const isDomain = des === hnDetails.domain;
+        const prettyDomainName = punycode.toUnicode(des);
+        const isPunycoded = prettyDomainName !== des;
+
+        const span = row.querySelector('span:first-of-type');
+        span.classList.toggle(
+            'isIDN',
+            isPunycoded &&
+                reCyrillicAmbiguous.test(prettyDomainName) === true &&
+                reCyrillicNonAmbiguous.test(prettyDomainName) === false
+        );
+        span.querySelector('span').textContent = prettyDomainName;
+        span.title = isDomain && isPunycoded ? des : '';
+
+        const classList = row.classList;
+        classList.toggle('isDomain', isDomain);
+        classList.toggle('isSubDomain', !isDomain);
+        classList.toggle('allowed', hnDetails.allowCount !== 0);
+        classList.toggle('blocked', hnDetails.blockCount !== 0);
+        classList.toggle('totalAllowed', hnDetails.totalAllowCount !== 0);
+        classList.toggle('totalBlocked', hnDetails.totalBlockCount !== 0);
+
+        row = row.nextElementSibling;
+    }
+
+    // Remove unused trailing rows
+    if ( row !== null ) {
+        while ( row.nextElementSibling !== null ) {
+            rowContainer.removeChild(row.nextElementSibling);
+        }
+        rowContainer.removeChild(row);
+    }
+
+    // Add new rows all at once
+    if ( toAppend.childElementCount !== 0 ) {
+        rowContainer.appendChild(toAppend);
     }
 
     if ( dfPaneBuilt !== true && popupData.advancedUserEnabled ) {
@@ -350,25 +367,21 @@ var buildAllFirewallRows = function() {
 
 /******************************************************************************/
 
-var renderPrivacyExposure = function() {
+const renderPrivacyExposure = function() {
     allDomains = {};
     allDomainCount = touchedDomainCount = 0;
     allHostnameRows = [];
 
     // Sort hostnames. First-party hostnames must always appear at the top
     // of the list.
-    var desHostnameDone = {};
-    var keys = Object.keys(popupData.firewallRules)
+    const desHostnameDone = {};
+    const keys = Object.keys(popupData.firewallRules)
                      .sort(rulekeyCompare);
-    var key, des, hnDetails;
-    for ( var i = 0; i < keys.length; i++ ) {
-        key = keys[i];
-        des = key.slice(2, key.indexOf(' ', 2));
+    for ( const key of keys ) {
+        const des = key.slice(2, key.indexOf(' ', 2));
         // Specific-type rules -- these are built-in
-        if ( des === '*' || desHostnameDone.hasOwnProperty(des) ) {
-            continue;
-        }
-        hnDetails = popupData.hostnameDict[des] || {};
+        if ( des === '*' || desHostnameDone.hasOwnProperty(des) ) { continue; }
+        const hnDetails = popupData.hostnameDict[des] || {};
         if ( allDomains.hasOwnProperty(hnDetails.domain) === false ) {
             allDomains[hnDetails.domain] = false;
             allDomainCount += 1;
@@ -383,35 +396,63 @@ var renderPrivacyExposure = function() {
         desHostnameDone[des] = true;
     }
 
-    var summary = domainsHitStr.replace('{{count}}', touchedDomainCount.toLocaleString())
-                               .replace('{{total}}', allDomainCount.toLocaleString());
+    const summary = domainsHitStr
+                    .replace('{{count}}', touchedDomainCount.toLocaleString())
+                    .replace('{{total}}', allDomainCount.toLocaleString());
     uDom.nodeFromId('popupHitDomainCount').textContent = summary;
 };
 
 /******************************************************************************/
 
+const updateHnSwitches = function() {
+    uDom.nodeFromId('no-popups').classList.toggle(
+        'on',
+        popupData.noPopups === true
+    );
+    uDom.nodeFromId('no-large-media').classList.toggle(
+        'on', popupData.noLargeMedia === true
+    );
+    uDom.nodeFromId('no-cosmetic-filtering').classList.toggle(
+        'on',
+        popupData.noCosmeticFiltering === true
+    );
+    uDom.nodeFromId('no-remote-fonts').classList.toggle(
+        'on',
+        popupData.noRemoteFonts === true
+    );
+    uDom.nodeFromId('no-scripting').classList.toggle(
+        'on',
+        popupData.noScripting === true
+    );
+};
+
+/******************************************************************************/
+
 // Assume everything has to be done incrementally.
 
-var renderPopup = function() {
-    var elem, text;
-
+const renderPopup = function() {
     if ( popupData.tabTitle ) {
         document.title = popupData.appName + ' - ' + popupData.tabTitle;
     }
 
-    elem = document.body;
-    elem.classList.toggle('advancedUser', popupData.advancedUserEnabled);
+    let elem = document.body;
+    elem.classList.toggle(
+        'advancedUser',
+        popupData.advancedUserEnabled === true
+    );
     elem.classList.toggle(
         'off',
-        popupData.pageURL === '' || !popupData.netFilteringSwitch
+        popupData.pageURL === '' || popupData.netFilteringSwitch !== true
     );
 
-    // If you think the `=== true` is pointless, you are mistaken
-    uDom.nodeFromId('gotoPick').classList.toggle('enabled', popupData.canElementPicker === true);
-    uDom.nodeFromId('gotoZap').classList.toggle('enabled', popupData.canElementPicker === true);
+    let canElementPicker = popupData.canElementPicker === true &&
+                           popupData.netFilteringSwitch === true;
+    uDom.nodeFromId('gotoPick').classList.toggle('enabled', canElementPicker);
+    uDom.nodeFromId('gotoZap').classList.toggle('enabled', canElementPicker);
 
-    var blocked = popupData.pageBlockedRequestCount,
-        total = popupData.pageAllowedRequestCount + blocked;
+    let blocked = popupData.pageBlockedRequestCount,
+        total = popupData.pageAllowedRequestCount + blocked,
+        text;
     if ( total === 0 ) {
         text = formatNumber(0);
     } else {
@@ -434,29 +475,26 @@ var renderPopup = function() {
     renderPrivacyExposure();
 
     // Extra tools
-    uDom.nodeFromId('no-popups').classList.toggle('on', popupData.noPopups === true);
-    uDom.nodeFromId('no-large-media').classList.toggle('on', popupData.noLargeMedia === true);
-    uDom.nodeFromId('no-cosmetic-filtering').classList.toggle('on', popupData.noCosmeticFiltering === true);
-    uDom.nodeFromId('no-remote-fonts').classList.toggle('on', popupData.noRemoteFonts === true);
+    updateHnSwitches();
 
     // Report blocked popup count on badge
     total = popupData.popupBlockedCount;
-    uDom.nodeFromSelector('#no-popups > span.badge')
-        .textContent = total ? total.toLocaleString() : '';
+    uDom.nodeFromSelector('#no-popups > span.fa-icon-badge')
+        .textContent = total ? Math.min(total, 99).toLocaleString() : '';
 
     // Report large media count on badge
     total = popupData.largeMediaCount;
-    uDom.nodeFromSelector('#no-large-media > span.badge')
-        .textContent = total ? total.toLocaleString() : '';
+    uDom.nodeFromSelector('#no-large-media > span.fa-icon-badge')
+        .textContent = total ? Math.min(total, 99).toLocaleString() : '';
 
     // Report remote font count on badge
     total = popupData.remoteFontCount;
-    uDom.nodeFromSelector('#no-remote-fonts > span.badge')
-        .textContent = total ? total.toLocaleString() : '';
+    uDom.nodeFromSelector('#no-remote-fonts > span.fa-icon-badge')
+        .textContent = total ? Math.min(total, 99).toLocaleString() : '';
 
     // https://github.com/chrisaljoudi/uBlock/issues/470
     // This must be done here, to be sure the popup is resized properly
-    var dfPaneVisible = popupData.dfEnabled;
+    const dfPaneVisible = popupData.dfEnabled;
 
     // https://github.com/chrisaljoudi/uBlock/issues/1068
     // Remember the last state of the firewall pane. This allows to
@@ -467,11 +505,20 @@ var renderPopup = function() {
         vAPI.localStorage.setItem('popupFirewallPane', dfPaneVisibleStored);
     }
 
-    uDom.nodeFromId('panes').classList.toggle('dfEnabled', dfPaneVisible);
+    uDom.nodeFromId('panes').classList.toggle(
+        'dfEnabled',
+        dfPaneVisible === true
+    );
 
     elem = uDom.nodeFromId('firewallContainer');
-    elem.classList.toggle('minimized', popupData.firewallPaneMinimized);
-    elem.classList.toggle('colorBlind', popupData.colorBlindFriendly);
+    elem.classList.toggle(
+        'minimized',
+        popupData.firewallPaneMinimized === true
+    );
+    elem.classList.toggle(
+        'colorBlind',
+        popupData.colorBlindFriendly === true
+    );
 
     // Build dynamic filtering pane only if in use
     if ( dfPaneVisible ) {
@@ -486,15 +533,14 @@ var renderPopup = function() {
 // https://github.com/gorhill/uBlock/issues/2889
 //   Use tooltip for ARIA purpose.
 
-var renderTooltips = function(selector) {
-    var elem, text;
-    for ( var entry of tooltipTargetSelectors ) {
+const renderTooltips = function(selector) {
+    for ( const entry of tooltipTargetSelectors ) {
         if ( selector !== undefined && entry[0] !== selector ) { continue; }
-        text = vAPI.i18n(
+        const text = vAPI.i18n(
             entry[1].i18n +
             (uDom.nodeFromSelector(entry[1].state) === null ? '1' : '2')
         );
-        elem = uDom.nodeFromSelector(entry[0]);
+        const elem = uDom.nodeFromSelector(entry[0]);
         elem.setAttribute('aria-label', text);
         elem.setAttribute('data-tip', text);
         if ( selector !== undefined ) {
@@ -504,7 +550,7 @@ var renderTooltips = function(selector) {
     }
 };
 
-var tooltipTargetSelectors = new Map([
+const tooltipTargetSelectors = new Map([
     [
         '#switch',
         {
@@ -539,14 +585,21 @@ var tooltipTargetSelectors = new Map([
             state: '#no-remote-fonts.on',
             i18n: 'popupTipNoRemoteFonts'
         }
-    ]
+    ],
+    [
+        '#no-scripting',
+        {
+            state: '#no-scripting.on',
+            i18n: 'popupTipNoScripting'
+        }
+    ],
 ]);
 
 /******************************************************************************/
 
 // All rendering code which need to be executed only once.
 
-var renderOnce = function() {
+let renderOnce = function() {
     renderOnce = function(){};
 
     if ( popupData.fontSize !== popupFontSize ) {
@@ -578,8 +631,8 @@ var renderOnce = function() {
     // scrollbar if ever its height is more than what is available.
     // For small displays: we use the whole viewport.
 
-    let rpane = uDom.nodeFromSelector('#panes > div:first-of-type'),
-        lpane = uDom.nodeFromSelector('#panes > div:last-of-type');
+    const rpane = uDom.nodeFromSelector('#panes > div:first-of-type');
+    const lpane = uDom.nodeFromSelector('#panes > div:last-of-type');
 
     lpane.style.setProperty('height', rpane.offsetHeight + 'px');
 
@@ -587,7 +640,7 @@ var renderOnce = function() {
     // viewport is not a perfect match for the popup panel.
 
     let resizeTimer;
-    let resize = function() {
+    const resize = function() {
         resizeTimer = undefined;
         // Do not use equality, fractional pixel dimension occurs and must
         // be ignored.
@@ -603,7 +656,7 @@ var renderOnce = function() {
         lpane.style.removeProperty('height');
         window.removeEventListener('resize', resizeAsync);
     };
-    let resizeAsync = function() {
+    const resizeAsync = function() {
         if ( resizeTimer !== undefined ) {
             clearTimeout(resizeTimer);
         }
@@ -615,22 +668,29 @@ var renderOnce = function() {
 
 /******************************************************************************/
 
-var renderPopupLazy = function() {
+const renderPopupLazy = function() {
     messaging.send(
         'popupPanel',
         { what: 'getPopupLazyData', tabId: popupData.tabId }
     );
 };
 
-var onPopupMessage = function(data) {
+const onPopupMessage = function(data) {
     if ( !data ) { return; }
     if ( data.tabId !== popupData.tabId ) { return; }
 
     switch ( data.what ) {
-    case 'cosmeticallyFilteredElementCountChanged':
-        var v = data.count || '';
-        uDom.nodeFromSelector('#no-cosmetic-filtering > span.badge')
-            .textContent = typeof v === 'number' ? v.toLocaleString() : v;
+    case 'domSurveyFinalReport':
+        let count = data.affectedElementCount || '';
+        uDom.nodeFromSelector('#no-cosmetic-filtering > span.fa-icon-badge')
+            .textContent = typeof count === 'number'
+                ? Math.min(count, 99).toLocaleString()
+                : count;
+        count = data.scriptCount || '';
+        uDom.nodeFromSelector('#no-scripting > span.fa-icon-badge')
+            .textContent = typeof count === 'number'
+                ? Math.min(count, 99).toLocaleString()
+                : count;
         break;
     }
 };
@@ -639,7 +699,7 @@ messaging.addChannelListener('popup', onPopupMessage);
 
 /******************************************************************************/
 
-var toggleNetFilteringSwitch = function(ev) {
+const toggleNetFilteringSwitch = function(ev) {
     if ( !popupData || !popupData.pageURL ) { return; }
     messaging.send(
         'popupPanel',
@@ -657,7 +717,7 @@ var toggleNetFilteringSwitch = function(ev) {
 
 /******************************************************************************/
 
-var gotoZap = function() {
+const gotoZap = function() {
     messaging.send(
         'popupPanel',
         {
@@ -672,7 +732,7 @@ var gotoZap = function() {
 
 /******************************************************************************/
 
-var gotoPick = function() {
+const gotoPick = function() {
     messaging.send(
         'popupPanel',
         {
@@ -686,14 +746,14 @@ var gotoPick = function() {
 
 /******************************************************************************/
 
-var gotoURL = function(ev) {
+const gotoURL = function(ev) {
     if ( this.hasAttribute('href') === false ) { return; }
 
     ev.preventDefault();
 
     let url = this.getAttribute('href');
     if (
-        url === 'logger-ui.html#tab_active' &&
+        url === 'logger-ui.html#_' &&
         typeof popupData.tabId === 'number'
     ) {
         url += '+' + popupData.tabId;
@@ -717,7 +777,7 @@ var gotoURL = function(ev) {
 
 /******************************************************************************/
 
-var toggleFirewallPane = function() {
+const toggleFirewallPane = function() {
     popupData.dfEnabled = !popupData.dfEnabled;
 
     messaging.send(
@@ -745,28 +805,27 @@ var toggleFirewallPane = function() {
 
 /******************************************************************************/
 
-var mouseenterCellHandler = function() {
+const mouseenterCellHandler = function() {
     if ( uDom(this).hasClass('ownRule') === false ) {
         dfHotspots.appendTo(this);
     }
 };
 
-var mouseleaveCellHandler = function() {
+const mouseleaveCellHandler = function() {
     dfHotspots.detach();
 };
 
 /******************************************************************************/
 
-var setFirewallRule = function(src, des, type, action, persist) {
+const setFirewallRule = function(src, des, type, action, persist) {
     // This can happen on pages where uBlock does not work
-    if ( typeof popupData.pageHostname !== 'string' || popupData.pageHostname === '' ) {
+    if (
+        typeof popupData.pageHostname !== 'string' ||
+        popupData.pageHostname === ''
+    ) {
         return;
     }
-    var onFirewallRuleChanged = function(response) {
-        cachePopupData(response);
-        updateAllFirewallCells();
-        hashFromPopupData();
-    };
+
     messaging.send(
         'popupPanel',
         {
@@ -779,18 +838,23 @@ var setFirewallRule = function(src, des, type, action, persist) {
             action: action,
             persist: persist
         },
-        onFirewallRuleChanged
+        response => {
+            cachePopupData(response);
+            updateAllFirewallCells();
+            hashFromPopupData();
+        }
     );
 };
 
 /******************************************************************************/
 
-var unsetFirewallRuleHandler = function(ev) {
-    var cell = uDom(this);
+const unsetFirewallRuleHandler = function(ev) {
+    const cell = ev.target;
+    const row = cell.closest('[data-des]');
     setFirewallRule(
-        cell.attr('data-src') === '/' ? '*' : popupData.pageHostname,
-        cell.attr('data-des'),
-        cell.attr('data-type'),
+        cell.getAttribute('data-src') === '/' ? '*' : popupData.pageHostname,
+        row.getAttribute('data-des'),
+        row.getAttribute('data-type'),
         0,
         ev.ctrlKey || ev.metaKey
     );
@@ -799,25 +863,23 @@ var unsetFirewallRuleHandler = function(ev) {
 
 /******************************************************************************/
 
-var setFirewallRuleHandler = function(ev) {
-    var hotspot = uDom(this);
-    var cell = hotspot.ancestors('[data-src]');
-    if ( cell.length === 0 ) {
-        return;
-    }
-    var action = 0;
-    var hotspotId = hotspot.attr('id');
-    if ( hotspotId === 'dynaAllow' ) {
+const setFirewallRuleHandler = function(ev) {
+    const hotspot = ev.target;
+    const cell = hotspot.closest('[data-src]');
+    if ( cell === null ) { return; }
+    const row = cell.closest('[data-des]');
+    let action = 0;
+    if ( hotspot.id === 'dynaAllow' ) {
         action = 2;
-    } else if ( hotspotId === 'dynaNoop' ) {
+    } else if ( hotspot.id === 'dynaNoop' ) {
         action = 3;
     } else {
         action = 1;
     }
     setFirewallRule(
-        cell.attr('data-src') === '/' ? '*' : popupData.pageHostname,
-        cell.attr('data-des'),
-        cell.attr('data-type'),
+        cell.getAttribute('data-src') === '/' ? '*' : popupData.pageHostname,
+        row.getAttribute('data-des'),
+        row.getAttribute('data-type'),
         action,
         ev.ctrlKey || ev.metaKey
     );
@@ -826,7 +888,7 @@ var setFirewallRuleHandler = function(ev) {
 
 /******************************************************************************/
 
-var reloadTab = function(ev) {
+const reloadTab = function(ev) {
     messaging.send(
         'popupPanel',
         {
@@ -850,7 +912,7 @@ var reloadTab = function(ev) {
 
 /******************************************************************************/
 
-var toggleMinimize = function(ev) {
+const toggleMinimize = function(ev) {
     // Special display mode: in its own tab/window, with no vertical restraint.
     // Useful to take snapshots of the whole list of domains -- example:
     //   https://github.com/gorhill/uBlock/issues/736#issuecomment-178879944
@@ -870,9 +932,9 @@ var toggleMinimize = function(ev) {
         return;
     }
 
-    popupData.firewallPaneMinimized = uDom.nodeFromId('firewallContainer')
-                                          .classList
-                                          .toggle('minimized');
+    popupData.firewallPaneMinimized =
+        uDom.nodeFromId('firewallContainer').classList.toggle('minimized');
+
     messaging.send(
         'popupPanel',
         {
@@ -886,7 +948,7 @@ var toggleMinimize = function(ev) {
 
 /******************************************************************************/
 
-var saveFirewallRules = function() {
+const saveFirewallRules = function() {
     messaging.send(
         'popupPanel',
         {
@@ -900,12 +962,7 @@ var saveFirewallRules = function() {
 
 /******************************************************************************/
 
-var revertFirewallRules = function() {
-    var onFirewallRuleChanged = function(response) {
-        cachePopupData(response);
-        updateAllFirewallCells();
-        hashFromPopupData();
-    };
+const revertFirewallRules = function() {
     messaging.send(
         'popupPanel',
         {
@@ -914,16 +971,21 @@ var revertFirewallRules = function() {
             desHostnames: popupData.hostnameDict,
             tabId: popupData.tabId
         },
-        onFirewallRuleChanged
+        response => {
+            cachePopupData(response);
+            updateAllFirewallCells();
+            updateHnSwitches();
+            hashFromPopupData();
+        }
     );
     uDom.nodeFromId('firewallContainer').classList.remove('dirty');
 };
 
 /******************************************************************************/
 
-var toggleHostnameSwitch = function(ev) {
-    var target = ev.currentTarget;
-    var switchName = target.getAttribute('id');
+const toggleHostnameSwitch = function(ev) {
+    const target = ev.currentTarget;
+    const switchName = target.getAttribute('id');
     if ( !switchName ) { return; }
     target.classList.toggle('on');
     messaging.send(
@@ -933,11 +995,16 @@ var toggleHostnameSwitch = function(ev) {
             name: switchName,
             hostname: popupData.pageHostname,
             state: target.classList.contains('on'),
-            tabId: popupData.tabId
+            tabId: popupData.tabId,
+            persist: popupData.dfEnabled === false || ev.ctrlKey || ev.metaKey
+        },
+        response => {
+            cachePopupData(response);
+            updateAllFirewallCells();
+            hashFromPopupData();
         }
     );
     renderTooltips('#' + switchName);
-    hashFromPopupData();
 };
 
 /******************************************************************************/
@@ -960,11 +1027,11 @@ var toggleHostnameSwitch = function(ev) {
 // on demand rather than forcing the main process to assume a client may need
 // it and thus having to push it all the time unconditionally.
 
-var pollForContentChange = (function() {
-    var pollTimer = null;
+const pollForContentChange = (function() {
+    let pollTimer;
 
-    var pollCallback = function() {
-        pollTimer = null;
+    const pollCallback = function() {
+        pollTimer = undefined;
         messaging.send(
             'popupPanel',
             {
@@ -976,7 +1043,7 @@ var pollForContentChange = (function() {
         );
     };
 
-    var queryCallback = function(response) {
+    const queryCallback = function(response) {
         if ( response ) {
             getPopupData(popupData.tabId);
             return;
@@ -984,10 +1051,8 @@ var pollForContentChange = (function() {
         poll();
     };
 
-    var poll = function() {
-        if ( pollTimer !== null ) {
-            return;
-        }
+    const poll = function() {
+        if ( pollTimer !== undefined ) { return; }
         pollTimer = vAPI.setTimeout(pollCallback, 1500);
     };
 
@@ -996,8 +1061,8 @@ var pollForContentChange = (function() {
 
 /******************************************************************************/
 
-var getPopupData = function(tabId) {
-    var onDataReceived = function(response) {
+const getPopupData = function(tabId) {
+    const onDataReceived = function(response) {
         cachePopupData(response);
         renderOnce();
         renderPopup();
@@ -1014,32 +1079,29 @@ var getPopupData = function(tabId) {
 
 /******************************************************************************/
 
-var onShowTooltip = function() {
-    if ( popupData.tooltipsDisabled ) {
-        return;
-    }
+const onShowTooltip = function(ev) {
+    if ( popupData.tooltipsDisabled ) { return; }
 
-    var target = this;
+    const target = ev.target;
 
     // Tooltip container
-    var ttc = uDom(target).ancestors('.tooltipContainer').nodeAt(0) ||
-              document.body;
-    var ttcRect = ttc.getBoundingClientRect();
+    const ttc = uDom(target).ancestors('.tooltipContainer').nodeAt(0) ||
+                document.body;
+    const ttcRect = ttc.getBoundingClientRect();
 
     // Tooltip itself
-    var tip = uDom.nodeFromId('tooltip');
+    const tip = uDom.nodeFromId('tooltip');
     tip.textContent = target.getAttribute('data-tip');
     tip.style.removeProperty('top');
     tip.style.removeProperty('bottom');
     ttc.appendChild(tip);
 
     // Target rect
-    var targetRect = target.getBoundingClientRect();
+    const targetRect = target.getBoundingClientRect();
 
     // Default is "over"
-    var pos;
-    var over = target.getAttribute('data-tip-position') !== 'under';
-    if ( over ) {
+    let pos;
+    if ( target.getAttribute('data-tip-position') !== 'under' ) {
         pos = ttcRect.height - targetRect.top + ttcRect.top;
         tip.style.setProperty('bottom', pos + 'px');
     } else {
@@ -1050,7 +1112,7 @@ var onShowTooltip = function() {
     tip.classList.add('show');
 };
 
-var onHideTooltip = function() {
+const onHideTooltip = function() {
     uDom.nodeFromId('tooltip').classList.remove('show');
 };
 
@@ -1065,7 +1127,7 @@ var onHideTooltip = function() {
     let tabId = null;
 
     // Extract the tab id of the page this popup is for
-    let matches = window.location.search.match(/[\?&]tabId=([^&]+)/);
+    const matches = window.location.search.match(/[\?&]tabId=([^&]+)/);
     if ( matches && matches.length === 2 ) {
         tabId = parseInt(matches[1], 10) || 0;
     }
