@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2017-2018 Raymond Hill
+    Copyright (C) 2017-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,18 +24,26 @@
 /******************************************************************************/
 
 µBlock.scriptletFilteringEngine = (function() {
-    let api = {};
-
-    let µb = µBlock,
-        scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB(),
+    const µb = µBlock,
         duplicates = new Set(),
-        acceptedCount = 0,
-        discardedCount = 0,
         scriptletCache = new µb.MRUCache(32),
-        exceptionsRegister = new Set(),
         scriptletsRegister = new Map(),
+        exceptionsRegister = new Set(),
         reEscapeScriptArg = /[\\'"]/g;
 
+    let acceptedCount = 0,
+        discardedCount = 0,
+        scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB();
+
+    const api = {
+        get acceptedCount() {
+            return acceptedCount;
+        },
+        get discardedCount() {
+            return discardedCount;
+        }
+    };
+
     // Purpose of `contentscriptCode` below is too programmatically inject
     // content script code which only purpose is to inject scriptlets. This
     // essentially does the same as what uBO's declarative content script does,
@@ -58,8 +66,8 @@
     // Consequently, the programmatic-injection code path is taken only with
     // Chromium-based browsers.
 
-    let contentscriptCode = (function() {
-        let parts = [
+    const contentscriptCode = (function() {
+        const parts = [
             '(',
             function(hostname, scriptlets) {
                 if (
@@ -163,15 +171,15 @@
         };
     })();
     
-    let lookupScriptlet = function(raw, reng, toInject) {
+    const lookupScriptlet = function(raw, reng, toInject) {
         if ( toInject.has(raw) ) { return; }
         if ( scriptletCache.resetTime < reng.modifyTime ) {
             scriptletCache.reset();
         }
-        var content = scriptletCache.lookup(raw);
+        let content = scriptletCache.lookup(raw);
         if ( content === undefined ) {
-            var token, args,
-                pos = raw.indexOf(',');
+            const pos = raw.indexOf(',');
+            let token, args;
             if ( pos === -1 ) {
                 token = raw;
             } else {
@@ -196,13 +204,12 @@
     // Fill template placeholders. Return falsy if:
     // - At least one argument contains anything else than /\w/ and `.`
 
-    let patchScriptlet = function(content, args) {
-        var i = 1,
-            pos, arg;
+    const patchScriptlet = function(content, args) {
+        let i = 1;
         while ( args !== '' ) {
-            pos = args.indexOf(',');
+            let pos = args.indexOf(',');
             if ( pos === -1 ) { pos = args.length; }
-            arg = args.slice(0, pos).trim().replace(reEscapeScriptArg, '\\$&');
+            const arg = args.slice(0, pos).trim().replace(reEscapeScriptArg, '\\$&');
             content = content.replace('{{' + i + '}}', arg);
             args = args.slice(pos + 1).trim();
             i++;
@@ -210,19 +217,19 @@
         return content;
     };
 
-    let logOne = function(isException, token, details) {
-        µb.logger.writeOne(
-            details.tabId,
-            'cosmetic',
-            {
+    const logOne = function(isException, token, details) {
+        µBlock.filteringContext
+            .duplicate()
+            .fromTabId(details.tabId)
+            .setRealm('cosmetic')
+            .setType('dom')
+            .setURL(details.url)
+            .setDocOriginFromURL(details.url)
+            .setFilter({
                 source: 'cosmetic',
-                raw: (isException ? '#@#' : '##') + 'script:inject(' + token + ')'
-            },
-            'dom',
-            details.url,
-            null,
-            details.hostname
-        );
+                raw: (isException ? '#@#' : '##') + '+js(' + token + ')'
+            })
+            .toLogger();
     };
 
     api.reset = function() {
@@ -244,7 +251,7 @@
 
         if ( parsed.hostnames.length === 0 ) {
             if ( parsed.exception ) {
-                writer.push([ 32, '!', '', parsed.suffix ]);
+                writer.push([ 32, 0 | 0b0001, '', parsed.suffix ]);
             }
             return;
         }
@@ -253,28 +260,26 @@
         //   Ignore instances of exception filter with negated hostnames,
         //   because there is no way to create an exception to an exception.
 
-        var µburi = µb.URI;
-
-        for ( var hostname of parsed.hostnames ) {
-            var negated = hostname.charCodeAt(0) === 0x7E /* '~' */;
+        for ( let hn of parsed.hostnames ) {
+            const negated = hn.charCodeAt(0) === 0x7E /* '~' */;
             if ( negated ) {
-                hostname = hostname.slice(1);
+                hn = hn.slice(1);
             }
-            var hash = µburi.domainFromHostname(hostname);
+            let hash = µb.staticExtFilteringEngine.compileHostnameToHash(hn);
             if ( parsed.exception ) {
                 if ( negated ) { continue; }
-                hash = '!' + hash;
+                hash |= 0b0001;
             } else if ( negated ) {
-                hash = '!' + hash;
+                hash |= 0b0001;
             }
-            writer.push([ 32, hash, hostname, parsed.suffix ]);
+            writer.push([ 32, hash, hn, parsed.suffix ]);
         }
     };
 
     // 01234567890123456789
-    // script:inject(token[, arg[, ...]])
-    //               ^                 ^
-    //              14                 -1
+    // +js(token[, arg[, ...]])
+    //     ^                 ^
+    //     4                -1
 
     api.fromCompiledContent = function(reader) {
         // 1001 = scriptlet injection
@@ -282,17 +287,17 @@
 
         while ( reader.next() ) {
             acceptedCount += 1;
-            var fingerprint = reader.fingerprint();
+            const fingerprint = reader.fingerprint();
             if ( duplicates.has(fingerprint) ) {
                 discardedCount += 1;
                 continue;
             }
             duplicates.add(fingerprint);
-            var args = reader.args();
+            const args = reader.args();
             if ( args.length < 4 ) { continue; }
             scriptletDB.add(
                 args[1],
-                { hostname: args[2], token: args[3].slice(14, -1) }
+                { hostname: args[2], token: args[3].slice(4, -1) }
             );
         }
     };
@@ -301,10 +306,10 @@
         if ( scriptletDB.size === 0 ) { return; }
         if ( µb.hiddenSettings.ignoreScriptInjectFilters ) { return; }
 
-        var reng = µb.redirectEngine;
+        const reng = µb.redirectEngine;
         if ( !reng ) { return; }
 
-        var hostname = request.hostname;
+        const hostname = request.hostname;
 
         // https://github.com/gorhill/uBlock/issues/2835
         //   Do not inject scriptlets if the site is under an `allow` rule.
@@ -315,16 +320,16 @@
             return;
         }
 
-        let domain = request.domain,
-            entity = request.entity;
+        const domain = request.domain;
+        const entity = request.entity;
 
         // https://github.com/gorhill/uBlock/issues/1954
         // Implicit
-        var hn = hostname;
+        let hn = hostname;
         for (;;) {
             lookupScriptlet(hn + '.js', reng, scriptletsRegister);
             if ( hn === domain ) { break; }
-            var pos = hn.indexOf('.');
+            const pos = hn.indexOf('.');
             if ( pos === -1 ) { break; }
             hn = hn.slice(pos + 1);
         }
@@ -334,12 +339,16 @@
 
         // Explicit
         let entries = [];
-        if ( domain !== '' ) {
-            scriptletDB.retrieve(domain, hostname, entries);
-            scriptletDB.retrieve(entity, entity, entries);
+        const domainHash = µb.staticExtFilteringEngine.makeHash(domain);
+        if ( domainHash !== 0 ) {
+            scriptletDB.retrieve(domainHash, hostname, entries);
         }
-        scriptletDB.retrieve('', hostname, entries);
-        for ( let entry of entries ) {
+        const entityHash = µb.staticExtFilteringEngine.makeHash(entity);
+        if ( entityHash !== 0 ) {
+            scriptletDB.retrieve(entityHash, entity, entries);
+        }
+        scriptletDB.retrieve(0, hostname, entries);
+        for ( const entry of entries ) {
             lookupScriptlet(entry.token, reng, scriptletsRegister);
         }
 
@@ -347,24 +356,26 @@
 
         // Collect exception filters.
         entries = [];
-        if ( domain !== '' ) {
-            scriptletDB.retrieve('!' + domain, hostname, entries);
-            scriptletDB.retrieve('!' + entity, entity, entries);
+        if ( domainHash !== 0 ) {
+            scriptletDB.retrieve(domainHash | 0b0001, hostname, entries);
         }
-        scriptletDB.retrieve('!', hostname, entries);
-        for ( let entry of entries ) {
+        if ( entityHash !== 0 ) {
+            scriptletDB.retrieve(entityHash | 0b0001, entity, entries);
+        }
+        scriptletDB.retrieve(0 | 0b0001, hostname, entries);
+        for ( const entry of entries ) {
             exceptionsRegister.add(entry.token);
         }
 
         // Return an array of scriptlets, and log results if needed. 
-        let out = [],
-            logger = µb.logger.isEnabled() ? µb.logger : null,
-            isException;
-        for ( let entry of scriptletsRegister ) {
-            if ( (isException = exceptionsRegister.has(entry[0])) === false ) {
+        const out = [];
+        const loggerEnabled = µb.logger.enabled;
+        for ( const entry of scriptletsRegister ) {
+            const isException = exceptionsRegister.has(entry[0]);
+            if ( isException === false ) {
                 out.push(entry[1]);
             }
-            if ( logger !== null ) {
+            if ( loggerEnabled ) {
                 logOne(isException, entry[0], request);
             }
         }
@@ -380,7 +391,7 @@
     api.injectNow = function(details) {
         if ( typeof details.frameId !== 'number' ) { return; }
         if ( µb.URI.isNetworkURI(details.url) === false ) { return; }
-        let request = {
+        const request = {
             tabId: details.tabId,
             frameId: details.frameId,
             url: details.url,
@@ -390,7 +401,7 @@
         };
         request.domain = µb.URI.domainFromHostname(request.hostname);
         request.entity = µb.URI.entityFromDomain(request.domain);
-        let scriptlets = µb.scriptletFilteringEngine.retrieve(request);
+        const scriptlets = µb.scriptletFilteringEngine.retrieve(request);
         if ( scriptlets === undefined ) { return; }
         let code = contentscriptCode.assemble(request.hostname, scriptlets);
         if ( µb.hiddenSettings.debugScriptlets ) {
@@ -415,19 +426,6 @@
         scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB(selfie);
     };
 
-    Object.defineProperties(api, {
-        acceptedCount: {
-            get: function() {
-                return acceptedCount;
-            }
-        },
-        discardedCount: {
-            get: function() {
-                return discardedCount;
-            }
-        }
-    });
-
     return api;
 })();
 
