@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,47 +29,58 @@
     if ( typeof callback !== 'function' ) {
         callback = this.noopFunc;
     }
-    var getBytesInUseHandler = function(bytesInUse) {
+    let bytesInUse;
+    let countdown = 0;
+
+    let process = count => {
+        if ( typeof count === 'number' ) {
+            if ( bytesInUse === undefined ) {
+                bytesInUse = 0;
+            }
+            bytesInUse += count;
+        }
+        countdown -= 1;
+        if ( countdown > 0 ) { return; }
         µBlock.storageUsed = bytesInUse;
         callback(bytesInUse);
     };
+
     // Not all platforms implement this method.
     if ( vAPI.storage.getBytesInUse instanceof Function ) {
-        vAPI.storage.getBytesInUse(null, getBytesInUseHandler);
-    } else {
+        countdown += 1;
+        vAPI.storage.getBytesInUse(null, process);
+    }
+    if (
+        this.cacheStorage !== vAPI.storage &&
+        this.cacheStorage.getBytesInUse instanceof Function
+    ) {
+        countdown += 1;
+        this.cacheStorage.getBytesInUse(null, process);
+    }
+    if ( countdown === 0 ) {
         callback();
     }
 };
 
 /******************************************************************************/
 
-µBlock.keyvalSetOne = function(key, val, callback) {
-    var bin = {};
-    bin[key] = val;
-    vAPI.storage.set(bin, callback || this.noopFunc);
-};
-
-/******************************************************************************/
-
 µBlock.saveLocalSettings = (function() {
-    var saveAfter = 4 * 60 * 1000;
+    let saveAfter = 4 * 60 * 1000;
 
-    var save = function() {
-        this.localSettingsLastSaved = Date.now();
-        vAPI.storage.set(this.localSettings);
-    };
-
-    var onTimeout = function() {
-        var µb = µBlock;
+    let onTimeout = ( ) => {
+        let µb = µBlock;
         if ( µb.localSettingsLastModified > µb.localSettingsLastSaved ) {
-            save.call(µb);
+            µb.saveLocalSettings();
         }
         vAPI.setTimeout(onTimeout, saveAfter);
     };
 
     vAPI.setTimeout(onTimeout, saveAfter);
 
-    return save;
+    return function(callback) {
+        this.localSettingsLastSaved = Date.now();
+        vAPI.storage.set(this.localSettings, callback);
+    };
 })();
 
 /******************************************************************************/
@@ -81,48 +92,25 @@
 /******************************************************************************/
 
 µBlock.loadHiddenSettings = function() {
-    var onLoaded = function(bin) {
+    vAPI.storage.get('hiddenSettings', bin => {
         if ( bin instanceof Object === false ) { return; }
-        var µb = µBlock,
-            hs = bin.hiddenSettings;
-        // Remove following condition once 1.15.12+ is widespread.
-        if (
-            hs instanceof Object === false &&
-            typeof bin.hiddenSettingsString === 'string'
-        ) {
-            vAPI.storage.remove('hiddenSettingsString');
-            hs = µBlock.hiddenSettingsFromString(bin.hiddenSettingsString);
-        }
+        let hs = bin.hiddenSettings;
         if ( hs instanceof Object ) {
-            var hsDefault = µb.hiddenSettingsDefault;
-            for ( var key in hsDefault ) {
+            let hsDefault = this.hiddenSettingsDefault;
+            for ( let key in hsDefault ) {
                 if (
                     hsDefault.hasOwnProperty(key) &&
                     hs.hasOwnProperty(key) &&
                     typeof hs[key] === typeof hsDefault[key]
                 ) {
-                    µb.hiddenSettings[key] = hs[key];
+                    this.hiddenSettings[key] = hs[key];
                 }
             }
-            // To remove once 1.15.26 is widespread. The reason is to ensure
-            // the change in the following commit is taken into account:
-            // https://github.com/gorhill/uBlock/commit/8071321e9104
-            if ( hs.manualUpdateAssetFetchPeriod === 2000 ) {
-                µb.hiddenSettings.manualUpdateAssetFetchPeriod =
-                    µb.hiddenSettingsDefault.manualUpdateAssetFetchPeriod;
-                hs.manualUpdateAssetFetchPeriod = undefined;
-                µb.saveHiddenSettings();
-            }
         }
         if ( vAPI.localStorage.getItem('immediateHiddenSettings') === null ) {
-            µb.saveImmediateHiddenSettings();
+            this.saveImmediateHiddenSettings();
         }
-    };
-
-    vAPI.storage.get(
-        [ 'hiddenSettings', 'hiddenSettingsString'],
-        onLoaded
-    );
+    });
 };
 
 // Note: Save only the settings which values differ from the default ones.
@@ -130,8 +118,8 @@
 // which were not modified by the user.
 
 µBlock.saveHiddenSettings = function(callback) {
-    var bin = { hiddenSettings: {} };
-    for ( var prop in this.hiddenSettings ) {
+    let bin = { hiddenSettings: {} };
+    for ( let prop in this.hiddenSettings ) {
         if (
             this.hiddenSettings.hasOwnProperty(prop) &&
             this.hiddenSettings[prop] !== this.hiddenSettingsDefault[prop]
@@ -198,8 +186,10 @@
     vAPI.localStorage.setItem(
         'immediateHiddenSettings',
         JSON.stringify({
+                  cacheStorageAPI: this.hiddenSettings.cacheStorageAPI,
+               disableWebAssembly: this.hiddenSettings.disableWebAssembly,
             suspendTabsUntilReady: this.hiddenSettings.suspendTabsUntilReady,
-            userResourcesLocation: this.hiddenSettings.userResourcesLocation
+            userResourcesLocation: this.hiddenSettings.userResourcesLocation,
         })
     );
 };
@@ -210,25 +200,33 @@
 /******************************************************************************/
 
 µBlock.savePermanentFirewallRules = function() {
-    this.keyvalSetOne('dynamicFilteringString', this.permanentFirewall.toString());
+    vAPI.storage.set({
+        dynamicFilteringString: this.permanentFirewall.toString()
+    });
 };
 
 /******************************************************************************/
 
 µBlock.savePermanentURLFilteringRules = function() {
-    this.keyvalSetOne('urlFilteringString', this.permanentURLFiltering.toString());
+    vAPI.storage.set({
+        urlFilteringString: this.permanentURLFiltering.toString()
+    });
 };
 
 /******************************************************************************/
 
 µBlock.saveHostnameSwitches = function() {
-    this.keyvalSetOne('hostnameSwitchesString', this.hnSwitches.toString());
+    vAPI.storage.set({
+        hostnameSwitchesString: this.permanentSwitches.toString()
+    });
 };
 
 /******************************************************************************/
 
 µBlock.saveWhitelist = function() {
-    this.keyvalSetOne('netWhitelist', this.stringFromWhitelist(this.netWhitelist));
+    vAPI.storage.set({
+        netWhitelist: this.stringFromWhitelist(this.netWhitelist)
+    });
     this.netWhitelistModifyTime = Date.now();
 };
 
@@ -407,37 +405,71 @@
 
 /******************************************************************************/
 
-µBlock.appendUserFilters = function(filters) {
+µBlock.appendUserFilters = function(filters, options) {
+    filters = filters.trim();
     if ( filters.length === 0 ) { return; }
 
-    var µb = this;
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/372
+    //   Auto comment using user-defined template.
+    let comment = '';
+    if (
+        options instanceof Object &&
+        options.autoComment === true &&
+        this.hiddenSettings.autoCommentFilterTemplate.indexOf('{{') !== -1
+    ) {
+        const d = new Date();
+        comment =
+            '! ' +
+            this.hiddenSettings.autoCommentFilterTemplate
+                .replace('{{date}}', d.toLocaleDateString())
+                .replace('{{time}}', d.toLocaleTimeString())
+                .replace('{{origin}}', options.origin);
+    }
 
-    var onSaved = function() {
-        var compiledFilters = µb.compileFilters(filters),
-            snfe = µb.staticNetFilteringEngine,
-            cfe = µb.cosmeticFilteringEngine,
-            acceptedCount = snfe.acceptedCount + cfe.acceptedCount,
-            discardedCount = snfe.discardedCount + cfe.discardedCount;
-        µb.applyCompiledFilters(compiledFilters, true);
-        var entry = µb.availableFilterLists[µb.userFiltersPath],
-            deltaEntryCount = snfe.acceptedCount + cfe.acceptedCount - acceptedCount,
-            deltaEntryUsedCount = deltaEntryCount - (snfe.discardedCount + cfe.discardedCount - discardedCount);
+    const onSaved = ( ) => {
+        const compiledFilters = this.compileFilters(
+            filters,
+            { assetKey: this.userFiltersPath }
+        );
+        const snfe = this.staticNetFilteringEngine;
+        const cfe = this.cosmeticFilteringEngine;
+        const acceptedCount = snfe.acceptedCount + cfe.acceptedCount;
+        const discardedCount = snfe.discardedCount + cfe.discardedCount;
+        this.applyCompiledFilters(compiledFilters, true);
+        const entry = this.availableFilterLists[this.userFiltersPath];
+        const deltaEntryCount =
+            snfe.acceptedCount +
+            cfe.acceptedCount - acceptedCount;
+        const deltaEntryUsedCount =
+            deltaEntryCount -
+            (snfe.discardedCount + cfe.discardedCount - discardedCount);
         entry.entryCount += deltaEntryCount;
         entry.entryUsedCount += deltaEntryUsedCount;
-        vAPI.storage.set({ 'availableFilterLists': µb.availableFilterLists });
-        µb.staticNetFilteringEngine.freeze();
-        µb.redirectEngine.freeze();
-        µb.staticExtFilteringEngine.freeze();
-        µb.selfieManager.destroy();
+        vAPI.storage.set({ 'availableFilterLists': this.availableFilterLists });
+        this.staticNetFilteringEngine.freeze();
+        this.redirectEngine.freeze();
+        this.staticExtFilteringEngine.freeze();
+        this.selfieManager.destroy();
     };
 
-    var onLoaded = function(details) {
+    const onLoaded = details => {
         if ( details.error ) { return; }
+        // The comment, if any, will be applied if and only if it is different
+        // from the last comment found in the user filter list.
+        if ( comment !== '' ) {
+            const pos = details.content.lastIndexOf(comment);
+            if (
+                pos === -1 ||
+                details.content.indexOf('\n!', pos + 1) !== -1
+            ) {
+                filters = '\n' + comment + '\n' + filters;
+            }
+        }
         // https://github.com/chrisaljoudi/uBlock/issues/976
-        // If we reached this point, the filter quite probably needs to be
-        // added for sure: do not try to be too smart, trying to avoid
-        // duplicates at this point may lead to more issues.
-        µb.saveUserFilters(details.content.trim() + '\n\n' + filters.trim(), onSaved);
+        //   If we reached this point, the filter quite probably needs to be
+        //   added for sure: do not try to be too smart, trying to avoid
+        //   duplicates at this point may lead to more issues.
+        this.saveUserFilters(details.content.trim() + '\n' + filters, onSaved);
     };
 
     this.loadUserFilters(onLoaded);
@@ -608,20 +640,18 @@
 
 µBlock.loadFilterLists = function(callback) {
     // Callers are expected to check this first.
-    if ( this.loadingFilterLists ) {
-        return;
-    }
+    if ( this.loadingFilterLists ) { return; }
     this.loadingFilterLists = true;
 
-    var µb = this,
-        filterlistsCount = 0,
-        loadedListKeys = [];
+    const µb = µBlock;
+    const loadedListKeys = [];
+    let filterlistsCount = 0;
 
     if ( typeof callback !== 'function' ) {
         callback = this.noopFunc;
     }
 
-    var onDone = function() {
+    const onDone = function() {
         µb.staticNetFilteringEngine.freeze();
         µb.staticExtFilteringEngine.freeze();
         µb.redirectEngine.freeze();
@@ -637,17 +667,19 @@
         callback();
 
         µb.selfieManager.destroy();
+        µb.lz4Codec.relinquish();
+
         µb.loadingFilterLists = false;
     };
 
-    var applyCompiledFilters = function(assetKey, compiled) {
-        var snfe = µb.staticNetFilteringEngine,
-            sxfe = µb.staticExtFilteringEngine,
-            acceptedCount = snfe.acceptedCount + sxfe.acceptedCount,
+    const applyCompiledFilters = function(assetKey, compiled) {
+        const snfe = µb.staticNetFilteringEngine;
+        const sxfe = µb.staticExtFilteringEngine;
+        let acceptedCount = snfe.acceptedCount + sxfe.acceptedCount,
             discardedCount = snfe.discardedCount + sxfe.discardedCount;
         µb.applyCompiledFilters(compiled, assetKey === µb.userFiltersPath);
         if ( µb.availableFilterLists.hasOwnProperty(assetKey) ) {
-            var entry = µb.availableFilterLists[assetKey];
+            const entry = µb.availableFilterLists[assetKey];
             entry.entryCount = snfe.acceptedCount + sxfe.acceptedCount -
                 acceptedCount;
             entry.entryUsedCount = entry.entryCount -
@@ -656,7 +688,7 @@
         loadedListKeys.push(assetKey);
     };
 
-    var onCompiledListLoaded = function(details) {
+    const onCompiledListLoaded = function(details) {
         applyCompiledFilters(details.assetKey, details.content);
         filterlistsCount -= 1;
         if ( filterlistsCount === 0 ) {
@@ -664,7 +696,7 @@
         }
     };
 
-    var onFilterListsReady = function(lists) {
+    const onFilterListsReady = function(lists) {
         µb.availableFilterLists = lists;
 
         µb.redirectEngine.reset();
@@ -677,8 +709,8 @@
         // because it *may* happens that some load operations are synchronous:
         // This happens for assets which do not exist, ot assets with no
         // content.
-        var toLoad = [];
-        for ( var assetKey in lists ) {
+        const toLoad = [];
+        for ( const assetKey in lists ) {
             if ( lists.hasOwnProperty(assetKey) === false ) { continue; }
             if ( lists[assetKey].off ) { continue; }
             toLoad.push(assetKey);
@@ -688,7 +720,7 @@
             return onDone();
         }
 
-        var i = toLoad.length;
+        let i = toLoad.length;
         while ( i-- ) {
             µb.getCompiledFilterList(toLoad[i], onCompiledListLoaded);
         }
@@ -707,7 +739,10 @@
 
     var onCompiledListLoaded2 = function(details) {
         if ( details.content === '' ) {
-            details.content = µb.compileFilters(rawContent);
+            details.content = µb.compileFilters(
+                rawContent,
+                { assetKey: assetKey }
+            );
             µb.assets.put(compiledPath, details.content);
         }
         rawContent = undefined;
@@ -747,27 +782,26 @@
 //   Lower minimum update period to 1 day.
 
 µBlock.extractFilterListMetadata = function(assetKey, raw) {
-    var listEntry = this.availableFilterLists[assetKey];
+    let listEntry = this.availableFilterLists[assetKey];
     if ( listEntry === undefined ) { return; }
     // Metadata expected to be found at the top of content.
-    var head = raw.slice(0, 1024),
-        matches, v;
+    let head = raw.slice(0, 1024);
     // https://github.com/gorhill/uBlock/issues/313
     // Always try to fetch the name if this is an external filter list.
     if ( listEntry.title === '' || listEntry.group === 'custom' ) {
-        matches = head.match(/(?:^|\n)(?:!|# )[\t ]*Title[\t ]*:([^\n]+)/i);
+        let matches = head.match(/(?:^|\n)(?:!|# )[\t ]*Title[\t ]*:([^\n]+)/i);
         if ( matches !== null ) {
             // https://bugs.chromium.org/p/v8/issues/detail?id=2869
-            // JSON.stringify/JSON.parse is to work around String.slice()
-            // potentially causing the whole raw filter list to be held in
-            // memory just because we cut out the title as a substring.
-            listEntry.title = JSON.parse(JSON.stringify(matches[1].trim()));
+            //   orphanizeString is to work around String.slice()
+            //   potentially causing the whole raw filter list to be held in
+            //   memory just because we cut out the title as a substring.
+            listEntry.title = this.orphanizeString(matches[1].trim());
         }
     }
     // Extract update frequency information
-    matches = head.match(/(?:^|\n)(?:!|# )[\t ]*Expires[\t ]*:[\t ]*(\d+)[\t ]*(h)?/i);
+    let matches = head.match(/(?:^|\n)(?:!|# )[\t ]*Expires[\t ]*:[\t ]*(\d+)[\t ]*(h)?/i);
     if ( matches !== null ) {
-        v = Math.max(parseInt(matches[1], 10), 1);
+        let v = Math.max(parseInt(matches[1], 10), 1);
         if ( matches[2] !== undefined ) {
             v = Math.ceil(v / 24);
         }
@@ -790,30 +824,37 @@
 
 /******************************************************************************/
 
-µBlock.compileFilters = function(rawText) {
-    var writer = new this.CompiledLineWriter();
+µBlock.compileFilters = function(rawText, details) {
+    let writer = new this.CompiledLineIO.Writer();
+
+    // Populate the writer with information potentially useful to the
+    // client compilers.
+    if ( details ) {
+        if ( details.assetKey ) {
+            writer.properties.set('assetKey', details.assetKey);
+        }
+    }
 
     // Useful references:
     //    https://adblockplus.org/en/filter-cheatsheet
     //    https://adblockplus.org/en/filters
-    var staticNetFilteringEngine = this.staticNetFilteringEngine,
-        staticExtFilteringEngine = this.staticExtFilteringEngine,
-        reIsWhitespaceChar = /\s/,
-        reMaybeLocalIp = /^[\d:f]/,
-        reIsLocalhostRedirect = /\s+(?:0\.0\.0\.0|broadcasthost|localhost|local|ip6-\w+)\b/,
-        reLocalIp = /^(?:0\.0\.0\.0|127\.0\.0\.1|::1|fe80::1%lo0)/,
-        line, c, pos,
-        lineIter = new this.LineIterator(this.processDirectives(rawText));
+    const staticNetFilteringEngine = this.staticNetFilteringEngine;
+    const staticExtFilteringEngine = this.staticExtFilteringEngine;
+    const reIsWhitespaceChar = /\s/;
+    const reMaybeLocalIp = /^[\d:f]/;
+    const reIsLocalhostRedirect = /\s+(?:0\.0\.0\.0|broadcasthost|localhost|local|ip6-\w+)\b/;
+    const reLocalIp = /^(?:0\.0\.0\.0|127\.0\.0\.1|::1|fe80::1%lo0)/;
+    const lineIter = new this.LineIterator(this.processDirectives(rawText));
 
     while ( lineIter.eot() === false ) {
         // rhill 2014-04-18: The trim is important here, as without it there
         // could be a lingering `\r` which would cause problems in the
         // following parsing code.
-        line = lineIter.next().trim();
+        let line = lineIter.next().trim();
         if ( line.length === 0 ) { continue; }
 
         // Strip comments
-        c = line.charAt(0);
+        const c = line.charAt(0);
         if ( c === '!' || c === '[' ) { continue; }
 
         // Parse or skip cosmetic filters
@@ -832,7 +873,7 @@
         // Don't remove:
         //   ...#blah blah blah
         // because some ABP filters uses the `#` character (URL fragment)
-        pos = line.indexOf('#');
+        const pos = line.indexOf('#');
         if ( pos !== -1 && reIsWhitespaceChar.test(line.charAt(pos - 1)) ) {
             line = line.slice(0, pos).trim();
         }
@@ -864,7 +905,7 @@
 
 µBlock.applyCompiledFilters = function(rawText, firstparty) {
     if ( rawText === '' ) { return; }
-    var reader = new this.CompiledLineReader(rawText);
+    let reader = new this.CompiledLineIO.Reader(rawText);
     this.staticNetFilteringEngine.fromCompiledContent(reader);
     this.staticExtFilteringEngine.fromCompiledContent(reader, {
         skipGenericCosmetic: this.userSettings.ignoreGenericCosmeticFilters,
@@ -967,38 +1008,31 @@
 
 /******************************************************************************/
 
-µBlock.loadPublicSuffixList = function(callback) {
-    var µb = this,
-        assetKey = µb.pslAssetKey,
-        compiledAssetKey = 'compiled/' + assetKey;
-
-    if ( typeof callback !== 'function' ) {
-        callback = this.noopFunc;
-    }
-    var onRawListLoaded = function(details) {
-        if ( details.content !== '' ) {
-            µb.compilePublicSuffixList(details.content);
-        }
-        callback();
-    };
-
-    var onCompiledListLoaded = function(details) {
-        var selfie;
+µBlock.loadPublicSuffixList = function() {
+    return new Promise(resolve => {
+    // start of executor
+    this.assets.get('compiled/' + this.pslAssetKey, details => {
+        let selfie;
         try {
             selfie = JSON.parse(details.content);
         } catch (ex) {
         }
         if (
-            selfie === undefined ||
-            publicSuffixList.fromSelfie(selfie) === false
+            selfie instanceof Object &&
+            publicSuffixList.fromSelfie(selfie)
         ) {
-            µb.assets.get(assetKey, onRawListLoaded);
+            resolve();
             return;
         }
-        callback();
-    };
-
-    this.assets.get(compiledAssetKey, onCompiledListLoaded);
+        this.assets.get(this.pslAssetKey, details => {
+            if ( details.content !== '' ) {
+                this.compilePublicSuffixList(details.content);
+            }
+            resolve();
+        });
+    });
+    // end of executor
+    });
 };
 
 /******************************************************************************/
@@ -1027,30 +1061,40 @@
 
     let create = function() {
         timer = null;
-        let selfie = {
+        let selfie = JSON.stringify({
             magic: µb.systemSettings.selfieMagic,
-            availableFilterLists: JSON.stringify(µb.availableFilterLists),
-            staticNetFilteringEngine: JSON.stringify(µb.staticNetFilteringEngine.toSelfie()),
-            redirectEngine: JSON.stringify(µb.redirectEngine.toSelfie()),
-            staticExtFilteringEngine: JSON.stringify(µb.staticExtFilteringEngine.toSelfie())
-        };
-        vAPI.cacheStorage.set({ selfie: selfie });
+            availableFilterLists: µb.availableFilterLists,
+            staticNetFilteringEngine: µb.staticNetFilteringEngine.toSelfie(),
+            redirectEngine: µb.redirectEngine.toSelfie(),
+            staticExtFilteringEngine: µb.staticExtFilteringEngine.toSelfie()
+        });
+        µb.cacheStorage.set({ selfie: selfie });
+        µb.lz4Codec.relinquish();
     };
 
     let load = function(callback) {
-        vAPI.cacheStorage.get('selfie', function(bin) {
+        µb.cacheStorage.get('selfie', function(bin) {
             if (
                 bin instanceof Object === false ||
-                bin.selfie instanceof Object === false ||
-                bin.selfie.magic !== µb.systemSettings.selfieMagic ||
-                bin.selfie.redirectEngine === undefined
+                typeof bin.selfie !== 'string'
             ) {
                 return callback(false);
             }
-            µb.availableFilterLists = JSON.parse(bin.selfie.availableFilterLists);
-            µb.staticNetFilteringEngine.fromSelfie(JSON.parse(bin.selfie.staticNetFilteringEngine));
-            µb.redirectEngine.fromSelfie(JSON.parse(bin.selfie.redirectEngine));
-            µb.staticExtFilteringEngine.fromSelfie(JSON.parse(bin.selfie.staticExtFilteringEngine));
+            let selfie;
+            try {
+                selfie = JSON.parse(bin.selfie);
+            } catch(ex) {
+            }
+            if (
+                selfie instanceof Object === false ||
+                selfie.magic !== µb.systemSettings.selfieMagic
+            ) {
+                return callback(false);
+            }
+            µb.availableFilterLists = selfie.availableFilterLists;
+            µb.staticNetFilteringEngine.fromSelfie(selfie.staticNetFilteringEngine);
+            µb.redirectEngine.fromSelfie(selfie.redirectEngine);
+            µb.staticExtFilteringEngine.fromSelfie(selfie.staticExtFilteringEngine);
             callback(true);
         });
     };
@@ -1060,7 +1104,7 @@
             clearTimeout(timer);
             timer = null;
         }
-        vAPI.cacheStorage.remove('selfie');
+        µb.cacheStorage.remove('selfie');
         timer = vAPI.setTimeout(create, µb.selfieAfter);
     };
 
@@ -1261,7 +1305,10 @@
                     );
                     this.assets.put(
                         'compiled/' + details.assetKey,
-                        this.compileFilters(details.content)
+                        this.compileFilters(
+                            details.content,
+                            { assetKey: details.assetKey }
+                        )
                     );
                 }
             } else {
@@ -1281,7 +1328,6 @@
             what: 'assetUpdated',
             key: details.assetKey,
             cached: cached
-            
         });
         // https://github.com/gorhill/uBlock/issues/2585
         // Whenever an asset is overwritten, the current selfie is quite
