@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -46,16 +46,16 @@ var µb = µBlock;
 /******************************************************************************/
 
 µb.normalizePageURL = function(tabId, pageURL) {
-    if ( vAPI.isBehindTheSceneTabId(tabId) ) {
+    if ( tabId < 0 ) {
         return 'http://behind-the-scene/';
     }
-    var uri = this.URI.set(pageURL);
-    var scheme = uri.scheme;
+    const uri = this.URI.set(pageURL);
+    const scheme = uri.scheme;
     if ( scheme === 'https' || scheme === 'http' ) {
         return uri.normalizedURI();
     }
 
-    var fakeHostname = scheme + '-scheme';
+    let fakeHostname = scheme + '-scheme';
 
     if ( uri.hostname !== '' ) {
         fakeHostname = uri.hostname + '.' + fakeHostname;
@@ -215,6 +215,7 @@ housekeep itself.
         this.stack = [];
         this.rawURL =
         this.normalURL =
+        this.origin =
         this.rootHostname =
         this.rootDomain = '';
         this.commitTimer = null;
@@ -305,14 +306,21 @@ housekeep itself.
     TabContext.prototype.update = function() {
         this.netFilteringReadTime = 0;
         if ( this.stack.length === 0 ) {
-            this.rawURL = this.normalURL = this.rootHostname = this.rootDomain = '';
+            this.rawURL =
+            this.normalURL =
+            this.origin =
+            this.rootHostname =
+            this.rootDomain = '';
             return;
         }
         var stackEntry = this.stack[this.stack.length - 1];
         this.rawURL = stackEntry.url;
         this.normalURL = µb.normalizePageURL(this.tabId, this.rawURL);
-        this.rootHostname = µb.URI.hostnameFromURI(this.normalURL);
-        this.rootDomain = µb.URI.domainFromHostname(this.rootHostname) || this.rootHostname;
+        this.origin = µb.URI.originFromURI(this.normalURL);
+        this.rootHostname = µb.URI.hostnameFromURI(this.origin);
+        this.rootDomain =
+            µb.URI.domainFromHostname(this.rootHostname) ||
+            this.rootHostname;
     };
 
     // Called whenever a candidate root URL is spotted for the tab.
@@ -431,15 +439,13 @@ housekeep itself.
 
     // Behind-the-scene tab context
     (function() {
-        var entry = new TabContext(vAPI.noTabId);
+        const entry = new TabContext(vAPI.noTabId);
         entry.stack.push(new StackEntry('', true));
         entry.rawURL = '';
         entry.normalURL = µb.normalizePageURL(entry.tabId);
-        entry.rootHostname = µb.URI.hostnameFromURI(entry.normalURL);
+        entry.origin = µb.URI.originFromURI(entry.normalURL);
+        entry.rootHostname = µb.URI.hostnameFromURI(entry.origin);
         entry.rootDomain = µb.URI.domainFromHostname(entry.rootHostname);
-
-        entry = new TabContext(vAPI.anyTabId);
-        entry.stack.push(new StackEntry('', true));
     })();
 
     // Context object, typically to be used to feed filtering engines.
@@ -454,6 +460,7 @@ housekeep itself.
         this.pageHostname =
         this.pageDomain =
         this.requestURL =
+        this.origin =
         this.requestHostname =
         this.requestDomain = '';
         return this;
@@ -550,8 +557,7 @@ vAPI.tabs.onClosed = function(tabId) {
 vAPI.tabs.onPopupUpdated = (function() {
     // The same context object will be reused everytime. This also allows to
     // remember whether a popup or popunder was matched.
-    var context = {},
-        logData;
+    const fctxt = µBlock.filteringContext.setFilter(undefined);
 
     // https://github.com/gorhill/uBlock/commit/1d448b85b2931412508aa01bf899e0b6f0033626#commitcomment-14944764
     //   See if two URLs are different, disregarding scheme -- because the
@@ -560,10 +566,10 @@ vAPI.tabs.onPopupUpdated = (function() {
     //   Maybe no link element was clicked.
     // https://github.com/gorhill/uBlock/issues/3287
     //   Do not bail out if the target URL has no hostname.
-    var areDifferentURLs = function(a, b) {
+    const areDifferentURLs = function(a, b) {
         if ( b === '' ) { return true; }
         if ( b.startsWith('about:') ) { return false; }
-        var pos = a.indexOf('://');
+        let pos = a.indexOf('://');
         if ( pos === -1 ) { return false; }
         a = a.slice(pos);
         pos = b.indexOf('://');
@@ -573,19 +579,12 @@ vAPI.tabs.onPopupUpdated = (function() {
         return b !== a;
     };
 
-    var popupMatch = function(openerURL, targetURL, popupType) {
-        var openerHostname = µb.URI.hostnameFromURI(openerURL),
-            openerDomain = µb.URI.domainFromHostname(openerHostname),
-            result;
-
-        context.pageHostname = openerHostname;
-        context.pageDomain = openerDomain;
-        context.rootURL = openerURL;
-        context.rootHostname = openerHostname;
-        context.rootDomain = openerDomain;
-        context.requestURL = targetURL;
-        context.requestHostname = µb.URI.hostnameFromURI(targetURL);
-        context.requestType = 'popup';
+    const popupMatch = function(openerURL, targetURL, popupType) {
+        fctxt.setTabOriginFromURL(openerURL)
+             .setDocOriginFromURL(openerURL)
+             .setURL(targetURL)
+             .setType('popup');
+        let result;
 
         // https://github.com/gorhill/uBlock/issues/1735
         //   Do not bail out on `data:` URI, they are commonly used for popups.
@@ -603,17 +602,20 @@ vAPI.tabs.onPopupUpdated = (function() {
         //   Ignore bad target URL. On Firefox, an `about:blank` tab may be
         //   opened for a new tab before it is filled in with the real target
         //   URL.
-        if ( openerHostname !== '' && targetURL !== 'about:blank' ) {
+        if ( fctxt.getTabHostname() !== '' && targetURL !== 'about:blank' ) {
             // Check per-site switch first
             // https://github.com/gorhill/uBlock/issues/3060
             // - The no-popups switch must apply only to popups, not to
             //   popunders.
             if (
                 popupType === 'popup' &&
-                µb.hnSwitches.evaluateZ('no-popups', openerHostname)
+                µb.sessionSwitches.evaluateZ(
+                    'no-popups',
+                    fctxt.getTabHostname()
+                )
             ) {
-                logData = {
-                    raw: 'no-popups: ' + µb.hnSwitches.z + ' true',
+                fctxt.filter = {
+                    raw: 'no-popups: ' + µb.sessionSwitches.z + ' true',
                     result: 1,
                     source: 'switch'
                 };
@@ -624,7 +626,7 @@ vAPI.tabs.onPopupUpdated = (function() {
             //   Take into account popup-specific rules in dynamic URL
             //   filtering, OR generic allow rules.
             result = µb.sessionURLFiltering.evaluateZ(
-                openerHostname,
+                fctxt.getTabHostname(),
                 targetURL,
                 popupType
             );
@@ -632,7 +634,7 @@ vAPI.tabs.onPopupUpdated = (function() {
                 result === 1 && µb.sessionURLFiltering.type === popupType ||
                 result === 2
             ) {
-                logData = µb.sessionURLFiltering.toLogData();
+                fctxt.filter = µb.sessionURLFiltering.toLogData();
                 return result;
             }
 
@@ -641,12 +643,12 @@ vAPI.tabs.onPopupUpdated = (function() {
             //   rules are ignored, as block rules are not meant to block
             //   specific types like `popup` (just like with static filters).
             result = µb.sessionFirewall.evaluateCellZY(
-                openerHostname,
-                context.requestHostname,
+                fctxt.getTabHostname(),
+                fctxt.getHostname(),
                 popupType
             );
             if ( result === 2 ) {
-                logData = µb.sessionFirewall.toLogData();
+                fctxt.filter = µb.sessionFirewall.toLogData();
                 return 2;
             }
         }
@@ -656,12 +658,11 @@ vAPI.tabs.onPopupUpdated = (function() {
         //   Don't block if uBlock is turned off in popup's context
         if ( µb.getNetFilteringSwitch(targetURL) ) {
             result = µb.staticNetFilteringEngine.matchStringExactType(
-                context,
-                targetURL,
+                fctxt,
                 popupType
             );
             if ( result !== 0 ) {
-                logData = µb.staticNetFilteringEngine.toLogData();
+                fctxt.filter = µb.staticNetFilteringEngine.toLogData();
                 return result;
             }
         }
@@ -669,23 +670,23 @@ vAPI.tabs.onPopupUpdated = (function() {
         return 0;
     };
 
-    var mapPopunderResult = function(popunderURL, popunderHostname, result) {
+    const mapPopunderResult = function(popunderURL, popunderHostname, result) {
         if (
-            logData === undefined ||
-            logData.source !== 'static' ||
-            logData.token === µb.staticNetFilteringEngine.noTokenHash
+            fctxt.filter === undefined ||
+            fctxt.filter !== 'static' ||
+            fctxt.filter.token === µb.staticNetFilteringEngine.noTokenHash
         ) {
             return 0;
         }
-        if ( logData.token === µb.staticNetFilteringEngine.dotTokenHash ) {
+        if ( fctxt.filter.token === µb.staticNetFilteringEngine.dotTokenHash ) {
             return result;
         }
-        var re = new RegExp(logData.regex, 'i'),
-            matches = re.exec(popunderURL);
+        const re = new RegExp(fctxt.filter.regex, 'i');
+        const matches = re.exec(popunderURL);
         if ( matches === null ) { return 0; }
-        var beg = matches.index,
-            end = beg + matches[0].length,
-            pos = popunderURL.indexOf(popunderHostname);
+        const beg = matches.index;
+        const end = beg + matches[0].length;
+        const pos = popunderURL.indexOf(popunderHostname);
         if ( pos === -1 ) { return 0; }
         // https://github.com/gorhill/uBlock/issues/1471
         //   We test whether the opener hostname as at least one character
@@ -698,36 +699,32 @@ vAPI.tabs.onPopupUpdated = (function() {
             : 0;
     };
 
-    var popunderMatch = function(openerURL, targetURL) {
-        var result = popupMatch(targetURL, openerURL, 'popunder');
-        if ( result === 1 ) {
-            return result;
-        }
+    const popunderMatch = function(openerURL, targetURL) {
+        let result = popupMatch(targetURL, openerURL, 'popunder');
+        if ( result === 1 ) { return result; }
+
         // https://github.com/gorhill/uBlock/issues/1010#issuecomment-186824878
         //   Check the opener tab as if it were the newly opened tab: if there
         //   is a hit against a popup filter, and if the matching filter is not
         //   a broad one, we will consider the opener tab to be a popunder tab.
         //   For now, a "broad" filter is one which does not touch any part of
         //   the hostname part of the opener URL.
-        var popunderURL = openerURL,
+        let popunderURL = openerURL,
             popunderHostname = µb.URI.hostnameFromURI(popunderURL);
-        if ( popunderHostname === '' ) {
-            return 0;
-        }
+        if ( popunderHostname === '' ) { return 0; }
+
         result = mapPopunderResult(
             popunderURL,
             popunderHostname,
             popupMatch(targetURL, popunderURL, 'popup')
         );
-        if ( result !== 0 ) {
-            return result;
-        }
+        if ( result !== 0 ) { return result; }
+
         // https://github.com/gorhill/uBlock/issues/1598
-        // Try to find a match against origin part of the opener URL.
+        //   Try to find a match against origin part of the opener URL.
         popunderURL = µb.URI.originFromURI(popunderURL);
-        if ( popunderURL === '' ) {
-            return 0;
-        }
+        if ( popunderURL === '' ) { return 0; }
+
         return mapPopunderResult(
             popunderURL,
             popunderHostname,
@@ -737,40 +734,43 @@ vAPI.tabs.onPopupUpdated = (function() {
 
     return function(targetTabId, openerDetails) {
         // Opener details.
-        var openerTabId = openerDetails.tabId;
-        var tabContext = µb.tabContextManager.lookup(openerTabId);
+        const openerTabId = openerDetails.tabId;
+        let tabContext = µb.tabContextManager.lookup(openerTabId);
         if ( tabContext === null ) { return; }
-        var openerURL = tabContext.rawURL;
+        const openerURL = tabContext.rawURL;
         if ( openerURL === '' ) { return; }
 
         // Popup details.
         tabContext = µb.tabContextManager.lookup(targetTabId);
         if ( tabContext === null ) { return; }
-        var targetURL = tabContext.rawURL;
+        let targetURL = tabContext.rawURL;
         if ( targetURL === '' ) { return; }
 
         // https://github.com/gorhill/uBlock/issues/341
         // Allow popups if uBlock is turned off in opener's context.
-        if ( µb.getNetFilteringSwitch(openerURL) === false ) {
-            return;
-        }
+        if ( µb.getNetFilteringSwitch(openerURL) === false ) { return; }
 
         // https://github.com/gorhill/uBlock/issues/1538
-        if ( µb.getNetFilteringSwitch(µb.normalizePageURL(openerTabId, openerURL)) === false ) {
+        if (
+            µb.getNetFilteringSwitch(µb.normalizePageURL(
+                openerTabId,
+                openerURL)
+            ) === false
+        ) {
             return;
         }
 
         // If the page URL is that of our "blocked page" URL, extract the URL of
         // the page which was blocked.
         if ( targetURL.startsWith(vAPI.getURL('document-blocked.html')) ) {
-            var matches = /details=([^&]+)/.exec(targetURL);
+            const matches = /details=([^&]+)/.exec(targetURL);
             if ( matches !== null ) {
                 targetURL = JSON.parse(atob(matches[1])).url;
             }
         }
 
         // Popup test.
-        var popupType = 'popup',
+        let popupType = 'popup',
             result = 0;
         // https://github.com/gorhill/uBlock/issues/2919
         // - If the target tab matches a clicked link, assume it's legit.
@@ -788,18 +788,21 @@ vAPI.tabs.onPopupUpdated = (function() {
 
         // Log only for when there was a hit against an actual filter (allow or block).
         // https://github.com/gorhill/uBlock/issues/2776
-        if ( µb.logger.isEnabled() ) {
-            µb.logger.writeOne(
-                popupType === 'popup' ? openerTabId : targetTabId,
-                'net',
-                result !== 0 ? logData : undefined,
-                popupType,
-                popupType === 'popup' ? targetURL : openerURL,
-                µb.URI.hostnameFromURI(context.rootURL),
-                µb.URI.hostnameFromURI(context.rootURL)
-            );
+        if ( µb.logger.enabled ) {
+            fctxt.setRealm('network').setType(popupType);
+            if ( popupType === 'popup' ) {
+                fctxt.setURL(targetURL)
+                     .setTabId(openerTabId)
+                     .setTabOriginFromURL(openerURL)
+                     .setDocOriginFromURL(openerURL);
+            } else {
+                fctxt.setURL(openerURL)
+                     .setTabId(targetTabId)
+                     .setTabOriginFromURL(targetURL)
+                     .setDocOriginFromURL(targetURL);
+            }
+            fctxt.toLogger();
         }
-        logData = undefined;
 
         // Not blocked
         if ( result !== 1 ) {
@@ -808,9 +811,9 @@ vAPI.tabs.onPopupUpdated = (function() {
 
         // Only if a popup was blocked do we report it in the dynamic
         // filtering pane.
-        var pageStore = µb.pageStoreFromTabId(openerTabId);
+        const pageStore = µb.pageStoreFromTabId(openerTabId);
         if ( pageStore ) {
-            pageStore.journalAddRequest(context.requestHostname, result);
+            pageStore.journalAddRequest(fctxt.getHostname(), result);
             pageStore.popupBlockedCount += 1;
         }
 
@@ -910,13 +913,9 @@ vAPI.tabs.registerListeners();
 // Permanent page store for behind-the-scene requests. Must never be removed.
 
 (function() {
-    var pageStore = µb.PageStore.factory(vAPI.noTabId);
+    const pageStore = µb.PageStore.factory(vAPI.noTabId);
     µb.pageStores.set(pageStore.tabId, pageStore);
     pageStore.title = vAPI.i18n('logBehindTheScene');
-
-    pageStore = µb.PageStore.factory(vAPI.anyTabId);
-    µb.pageStores.set(pageStore.tabId, pageStore);
-    pageStore.title = '[Any one of the known tabs]';
 })();
 
 /******************************************************************************/
