@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2018 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ var reRFC3986 = /^([^:\/?#]+:)?(\/\/[^\/?#]*)?([^?#]*)(\?[^#]*)?(#.*)?/;
 // Derived
 var reSchemeFromURI          = /^[^:\/?#]+:/;
 var reAuthorityFromURI       = /^(?:[^:\/?#]+:)?(\/\/[^\/?#]+)/;
-var reOriginFromURI          = /^(?:[^:\/?#]+:)\/\/(?:[^\/?#]+)?/;
+var reOriginFromURI          = /^(?:[^:\/?#]+:)\/\/[^\/?#]+/;
 var reCommonHostnameFromURL  = /^https?:\/\/([0-9a-z_][0-9a-z._-]*[0-9a-z])\//;
 var rePathFromURI            = /^(?:[^:\/?#]+:)?(?:\/\/[^\/?#]*)?([^?#]*)/;
 var reMustNormalizeHostname  = /[^0-9a-z._-]/;
@@ -225,7 +225,7 @@ URI.assemble = function(bits) {
 /******************************************************************************/
 
 URI.originFromURI = function(uri) {
-    var matches = reOriginFromURI.exec(uri);
+    const matches = reOriginFromURI.exec(uri);
     return matches !== null ? matches[0].toLowerCase() : '';
 };
 
@@ -330,39 +330,40 @@ URI.pathFromURI = function(uri) {
 // specific set of hostnames within a narrow time span -- in other words, I
 // believe probability of cache hit are high in uBlock.
 
-var domainCache = new Map();
-var domainCacheCountLowWaterMark = 40;
-var domainCacheCountHighWaterMark = 60;
-var domainCacheEntryJunkyardMax =
+const domainCache = new Map();
+const domainCacheCountLowWaterMark = 40;
+const domainCacheCountHighWaterMark = 60;
+const domainCacheEntryJunkyardMax =
     domainCacheCountHighWaterMark - domainCacheCountLowWaterMark;
 
-var DomainCacheEntry = function(domain) {
+const DomainCacheEntry = function(domain) {
     this.init(domain);
 };
 
-DomainCacheEntry.prototype.init = function(domain) {
-    this.domain = domain;
-    this.tstamp = Date.now();
-    return this;
+DomainCacheEntry.prototype = {
+    init: function(domain) {
+        this.domain = domain;
+        this.tstamp = Date.now();
+        return this;
+    },
+    dispose: function() {
+        this.domain = '';
+        if ( domainCacheEntryJunkyard.length < domainCacheEntryJunkyardMax ) {
+            domainCacheEntryJunkyard.push(this);
+        }
+    },
 };
 
-DomainCacheEntry.prototype.dispose = function() {
-    this.domain = '';
-    if ( domainCacheEntryJunkyard.length < domainCacheEntryJunkyardMax ) {
-        domainCacheEntryJunkyard.push(this);
-    }
-};
-
-var domainCacheEntryFactory = function(domain) {
+const domainCacheEntryFactory = function(domain) {
     return domainCacheEntryJunkyard.length !== 0 ?
         domainCacheEntryJunkyard.pop().init(domain) :
         new DomainCacheEntry(domain);
 };
 
-var domainCacheEntryJunkyard = [];
+const domainCacheEntryJunkyard = [];
 
-var domainCacheAdd = function(hostname, domain) {
-    var entry = domainCache.get(hostname);
+const domainCacheAdd = function(hostname, domain) {
+    const entry = domainCache.get(hostname);
     if ( entry !== undefined ) {
         entry.tstamp = Date.now();
     } else {
@@ -374,17 +375,17 @@ var domainCacheAdd = function(hostname, domain) {
     return domain;
 };
 
-var domainCacheEntrySort = function(a, b) {
+const domainCacheEntrySort = function(a, b) {
     return domainCache.get(b).tstamp - domainCache.get(a).tstamp;
 };
 
-var domainCachePrune = function() {
-    var hostnames = Array.from(domainCache.keys())
-                         .sort(domainCacheEntrySort)
-                         .slice(domainCacheCountLowWaterMark);
-    var i = hostnames.length;
+const domainCachePrune = function() {
+    const hostnames = Array.from(domainCache.keys())
+                           .sort(domainCacheEntrySort)
+                           .slice(domainCacheCountLowWaterMark);
+    let i = hostnames.length;
     while ( i-- ) {
-        var hostname = hostnames[i];
+        const hostname = hostnames[i];
         domainCache.get(hostname).dispose();
         domainCache.delete(hostname);
     }
@@ -409,7 +410,7 @@ URI.isNetworkURI = function(uri) {
     return reNetworkURI.test(uri);
 };
 
-var reNetworkURI = /^(?:ftps?|https?|wss?):\/\//;
+const reNetworkURI = /^(?:ftps?|https?|wss?):\/\//;
 
 /******************************************************************************/
 
@@ -417,7 +418,7 @@ URI.isNetworkScheme = function(scheme) {
     return reNetworkScheme.test(scheme);
 };
 
-var reNetworkScheme = /^(?:ftps?|https?|wss?)$/;
+const reNetworkScheme = /^(?:ftps?|https?|wss?)$/;
 
 /******************************************************************************/
 
