@@ -5,7 +5,8 @@
 echo "*** uBlock0.firefox: Creating web store package"
 echo "*** uBlock0.firefox: Copying files"
 
-DES=dist/build/uBlock0.firefox
+BLDIR=dist/build
+DES="$BLDIR"/uBlock0.firefox
 rm -rf $DES
 mkdir -p $DES
 
@@ -40,8 +41,6 @@ rm $DES/js/vapi-usercss.pseudo.js
 
 # Firefox/webext-specific
 rm $DES/img/icon_128.png
-rm $DES/options_ui.html
-rm $DES/js/options_ui.js
 
 echo "*** uBlock0.firefox: Generating web accessible resources..."
 cp -R src/web_accessible_resources $DES/
@@ -55,6 +54,12 @@ if [ "$1" = all ]; then
     pushd $DES > /dev/null
     zip ../$(basename $DES).xpi -qr *
     popd > /dev/null
+elif [ -n "$1" ]; then
+    echo "*** uBlock0.firefox: Creating versioned package..."
+    pushd $DES > /dev/null
+    zip ../$(basename $DES).xpi -qr *
+    popd > /dev/null
+    mv "$BLDIR"/uBlock0.firefox.xpi "$BLDIR"/uBlock0_"$1".firefox.xpi
 fi
 
 echo "*** uBlock0.firefox: Package done."
