@@ -2,6 +2,9 @@
 #
 # This script assumes a linux environment
 
+# https://github.com/uBlockOrigin/uBlock-issues/issues/217
+set -e
+
 echo "*** uBlock0.webext: Creating web store package"
 echo "*** uBlock0.webext: Copying files"
 
@@ -25,7 +28,13 @@ cp LICENSE.txt                          $DES/
 
 cp platform/webext/manifest.json        $DES/
 cp platform/webext/vapi-usercss.js      $DES/js/
-cp platform/webext/vapi-webrequest.js   $DES/js/
+
+# https://github.com/uBlockOrigin/uBlock-issues/issues/407
+echo "*** uBlock0.webext: concatenating vapi-webrequest.js"
+cat platform/chromium/vapi-webrequest.js > /tmp/vapi-webrequest.js
+echo >> /tmp/contentscript.js
+grep -v "^'use strict';$" platform/firefox/vapi-webrequest.js >> /tmp/vapi-webrequest.js
+mv /tmp/vapi-webrequest.js $DES/js/vapi-webrequest.js
 
 echo "*** uBlock0.webext: concatenating content scripts"
 cat $DES/js/vapi-usercss.js > /tmp/contentscript.js
@@ -45,13 +54,18 @@ cp -R src/web_accessible_resources $DES/
 python3 tools/import-war.py $DES/
 
 echo "*** uBlock0.webext: Generating meta..."
-python tools/make-webext-meta.py $DES/
+python3 tools/make-webext-meta.py $DES/
 
 if [ "$1" = all ]; then
     echo "*** uBlock0.webext: Creating package..."
     pushd $DES > /dev/null
     zip ../$(basename $DES).xpi -qr *
     popd > /dev/null
+elif [ -n "$1" ]; then
+    echo "*** uBlock0.webext: Creating versioned package..."
+    pushd $DES > /dev/null
+    zip ../$(basename $DES).xpi -qr * -O ../uBlock0_"$1".webext.xpi
+    popd > /dev/null
 fi
 
 echo "*** uBlock0.webext: Package done."
